/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_assert.h
 *
 * Purpose:     Compiler discrimination for the recls API.
 *
 * Created:     15th August 2003
 * Updated:     6th March 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLS_ASSERT
#define RECLS_INCL_H_RECLS_ASSERT

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_ASSERT_MAJOR     2
# define RECLS_VER_H_RECLS_ASSERT_MINOR     0
# define RECLS_VER_H_RECLS_ASSERT_REVISION  4
# define RECLS_VER_H_RECLS_ASSERT_EDIT      13
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_assert.h Assertions for the \ref group_recls API */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"

/** \def recls_assert Assert macro for the recls API
 *
 * \param x The expression that must evaluate to \c true
 */

#if defined(RECLS_PLATFORM_IS_WIN32) && \
    defined(_MSC_VER) && \
    (   defined(_DEBUG) || \
        defined(DEBUG))
# include <crtdbg.h> /* Prefer MSVCRT for VC++ and compatible compilers */
# define recls_assert(x)    _ASSERTE(x)
#else
# include <assert.h>
# define recls_assert(x)    assert(x)
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 */

/** \def recls_message_assert Assert macro for the recls API
 *
 * \param m The literal string describing the failed condition
 * \param x The expression that must evaluate to \c true
 */

#if defined(__WATCOMC__)
# define recls_message_assert(m, x)     recls_assert(x)
#else
# define recls_message_assert(m, x)     recls_assert((m, x))
#endif /* __WATCOMC__ */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLS_ASSERT */

/* ////////////////////////////////////////////////////////////////////////// */
