/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_debug.h
 *
 * Purpose:     Compiler discrimination for the recls API.
 *
 * Created:     30th September 2003
 * Updated:     6th March 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLS_DEBUG
#define RECLS_INCL_H_RECLS_DEBUG

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_DEBUG_MAJOR      2
# define RECLS_VER_H_RECLS_DEBUG_MINOR      0
# define RECLS_VER_H_RECLS_DEBUG_REVISION   3
# define RECLS_VER_H_RECLS_DEBUG_EDIT       13
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_debug.h Debug infrastructure for the \ref group_recls API */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#if 0
        #if !defined(_DEBUG) && defined(RECLS_PLATFORM_IS_WIN32)
        # define _DEBUG
        #endif /* _DEBUG */
#endif /* 0 */

#include "recls.h"
#if defined(RECLS_PLATFORM_IS_WIN32) && \
    defined(_DEBUG)
# include <winstl_error_scope.h>
# include <winstl_tls_index.h>
#endif /* _DEBUG && RECLS_PLATFORM_IS_WIN32 */
#include <stdio.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

#if defined(_DEBUG) && \
    defined(RECLS_PLATFORM_IS_WIN32)
winstl_ns_using(last_error_scope)
winstl_ns_using(tss_index)
#endif /* _DEBUG && RECLS_PLATFORM_IS_WIN32 */

/* /////////////////////////////////////////////////////////////////////////////
 * debug_printf
 */

#if defined(_DEBUG) && \
    defined(RECLS_PLATFORM_IS_WIN32)

inline void debug_printf(char const *fmt, ...)
{
    va_list args;
    char    _sz[2048];

    va_start(args, fmt);

    _vsnprintf(_sz, RECLS_NUM_ELEMENTS(_sz), fmt, args);
    OutputDebugStringA(_sz);

    va_end(args);
}

class function_scope
{
public:
    function_scope(char const *fn)
    {
        last_error_scope    error_scope;

        strncpy(m_fn, fn, RECLS_NUM_ELEMENTS(m_fn) - 1);
        debug_printf("%*s>> %s()\n", (int)post_inc_(), "", m_fn);
    }
    ~function_scope()
    {
        last_error_scope    error_scope;

        debug_printf("%*s<< %s()\n", (int)pre_inc_(), "", m_fn);
    }

private:
    typedef ::stlsoft::sint32_t int32_t;

#if defined(RECLS_COMPILER_IS_INTEL)
# pragma warning(push)
# pragma warning(disable : 171)
#endif /* RECLS_COMPILER_IS_INTEL */

	static int32_t tls_get_value_()
	{
		union
		{
			int32_t	i;
			void	*pv;
		} u;

		u.pv = ::TlsGetValue(sm_index);

		return u.i;
	}
	static void tls_set_value_(int32_t i)
	{
		union
		{
			int32_t	i;
			void	*pv;
		} u;

		u.i = i;

		::TlsSetValue(sm_index, u.pv);
	}

    static int32_t  post_inc_()
    {
//        int32_t i = reinterpret_cast<int32_t>(::TlsGetValue(sm_index));
        int32_t i = (int32_t)(tls_get_value_());
        int32_t r = i++;

        tls_set_value_(i);

        return r;
    }
    static int32_t  pre_inc_()
    {
//        int32_t i = reinterpret_cast<int32_t>(::TlsGetValue(sm_index));
        int32_t i = (int32_t)(tls_get_value_());
        int32_t r = --i;

        tls_set_value_(i);

        return r;
    }

#if defined(RECLS_COMPILER_IS_INTEL)
# pragma warning(pop)
#endif /* RECLS_COMPILER_IS_INTEL */

private:
    char                m_fn[1024];
    static tss_index    sm_index;
};

# define function_scope_trace(f)        function_scope  _scope_ ## __LINE__(f)

#else /* ? _DEBUG && RECLS_PLATFORM_IS_WIN32 */
# ifdef __cplusplus
inline 
# else /* ? __cplusplus */
static 
# endif /* __cplusplus */
       void debug_printf_(char const *fmt, ...)
{
    ((void)fmt);
}
# define debug_printf                   (0) ? ((void)0) : debug_printf_
# define function_scope_trace(f)        do { ; } while(0)
#endif /* _DEBUG && RECLS_PLATFORM_IS_WIN32 */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLS_DEBUG */

/* ////////////////////////////////////////////////////////////////////////// */
