/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_internal.h
 *
 * Purpose:     Main header file for the recls API.
 *
 * Created:     15th August 2003
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLS_INTERNAL
#define RECLS_INCL_H_RECLS_INTERNAL

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_INTERNAL_MAJOR       2
# define RECLS_VER_H_RECLS_INTERNAL_MINOR       0
# define RECLS_VER_H_RECLS_INTERNAL_REVISION    2
# define RECLS_VER_H_RECLS_INTERNAL_EDIT        27
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef __cplusplus
# error This file can only be included in C++ compilation units
#endif /* __cplusplus */

#include "recls.h"

/* /////////////////////////////////////////////////////////////////////////////
 * Compiler / language features
 *
 * Note: these should be moved out to the compiler-specific files later
 */

#if defined(RECLS_COMPILER_IS_COMO) || \
    defined(RECLS_COMPILER_IS_GCC) || \
    defined(RECLS_COMPILER_IS_INTEL) || \
    (   defined(RECLS_COMPILER_IS_MSVC) && \
        _MSC_VER > 1100) || \
    defined(RECLS_COMPILER_IS_MWERKS)
# define RECLS_COMPILER_SUPPORTS_PLACEMENT_DELETE
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Enums
 */

namespace SearchProtocol
{
    enum SearchProtocol
    {
            unknown         =   -1
        ,   fileSystem      =   0
        ,   ftp             =   1
        ,   win32Registry   =   2
    };
} // namespace SearchProtocol

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 */

#ifndef RECLS_NUM_ELEMENTS
# if defined(stlsoft_num_elements)
#  define RECLS_NUM_ELEMENTS(x)             stlsoft_num_elements(x)
# else /* ? stlsoft_num_elements */
#  define RECLS_NUM_ELEMENTS(x)             (sizeof(x) / sizeof((x)[0]))
# endif /* stlsoft_num_elements */
#endif /* !RECLS_NUM_ELEMENTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Interfaces
 */

#if 0
// class ReclsDNode
/// Interface for directory nodes
///
/// \note It has an ugly name-prefix if need to compile with compiler that does not support namespaces
struct ReclsDNode
{
// Construction
public:
    /// Destructor
    ///
    /// ReclsDNode instances are <b>not</b> reference-counted, but are
    /// deleted by their owner. They are non-shareable.
    virtual ~ReclsDNode() = 0;

// Operations
public:
    virtual recls_rc_t GetNext() = 0;

    virtual recls_rc_t GetDetails(recls_info_t *pinfo) = 0;

    virtual recls_rc_t GetNextDetails(recls_info_t *pinfo) = 0;
};

inline ReclsDNode::~ReclsDNode()
{}
#endif /* 0 */

// class ReclsSearch
struct ReclsSearch
{
// Construction
public:
    /// Destructor
    ///
    /// ReclsSearch instances are <b>not</b> reference-counted, but are
    /// deleted by their owner. They are non-shareable.
    virtual ~ReclsSearch() = 0;

// Operations
public:
    virtual recls_rc_t GetNext() = 0;

    virtual recls_rc_t GetDetails(recls_info_t *pinfo) = 0;

    virtual recls_rc_t GetNextDetails(recls_info_t *pinfo) = 0;

// Accessors
public:
    virtual recls_rc_t  GetLastError() const = 0;

// Handle interconversion
public:
    static hrecls_t     ToHandle(ReclsSearch *si);
    static ReclsSearch  *FromHandle(hrecls_t h);
};

inline ReclsSearch::~ReclsSearch()
{}

inline /* static */ hrecls_t ReclsSearch::ToHandle(ReclsSearch *si)
{
    return hrecls_t(si);
}

inline /* static */ ReclsSearch *ReclsSearch::FromHandle(hrecls_t h)
{
    return static_cast<ReclsSearch*>(const_cast<void *>(static_cast<void const *>(h)));
}

/* /////////////////////////////////////////////////////////////////////////////
 * Helper functions
 */

RECLS_FNDECL(size_t) Recls_GetStringProperty_(  struct recls_strptrs_t const    *ptrs
                                            ,   recls_char_t                    *buffer
                                            ,   size_t                          cchBuffer);

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLS_INTERNAL */

/* ////////////////////////////////////////////////////////////////////////// */
