/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_util.h
 *
 * Purpose:     Utility functions for the recls API.
 *
 * Created:     17th August 2003
 * Updated:     6th March 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLS_UTIL
#define RECLS_INCL_H_RECLS_UTIL

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_UTIL_MAJOR       2
# define RECLS_VER_H_RECLS_UTIL_MINOR       1
# define RECLS_VER_H_RECLS_UTIL_REVISION    1
# define RECLS_VER_H_RECLS_UTIL_EDIT        16
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_util.h Utility functions for the \ref group_recls  API */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

RECLS_LINKAGE_C recls_bool_t is_dots(recls_char_t const *f);

RECLS_LINKAGE_C size_t      align_up_size(size_t i);

RECLS_LINKAGE_C size_t      count_char_instances_a(recls_char_a_t const *begin, recls_char_a_t const *end, recls_char_a_t const ch);
RECLS_LINKAGE_C size_t      count_char_instances_w(recls_char_w_t const *begin, recls_char_w_t const *end, recls_char_w_t const ch);

#ifdef __cplusplus
inline size_t count_char_instances(recls_char_a_t const *begin, recls_char_a_t const *end, recls_char_a_t const ch)
{
    return count_char_instances_a(begin, end, ch);
}

inline size_t count_char_instances(recls_char_w_t const *begin, recls_char_w_t const *end, recls_char_a_t const ch)
{
    return count_char_instances_w(begin, end, ch);
}
#endif /* __cplusplus */

#if 0
RECLS_LINKAGE_C size_t      count_dir_parts_a(recls_char_a_t const *begin, recls_char_a_t const *end);
RECLS_LINKAGE_C size_t      count_dir_parts_w(recls_char_w_t const *begin, recls_char_w_t const *end);

#ifdef __cplusplus
inline size_t count_dir_parts(recls_char_a_t const *begin, recls_char_a_t const *end)
{
    return count_dir_parts_a(begin, end);
}

inline size_t count_dir_parts(recls_char_w_t const *begin, recls_char_w_t const *end)
{
    return count_dir_parts_w(begin, end);
}
#endif /* __cplusplus */
#endif /* 0 */

RECLS_LINKAGE_C size_t      recls_strncpy(recls_char_t *dest, size_t cchDest, recls_char_t const *src, size_t cchSrc);
RECLS_LINKAGE_C size_t      recls_strlen(recls_char_t const *dest);

RECLS_LINKAGE_C recls_char_t const *find_directory_0(recls_char_t const *path);

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLS_UTIL */

/* ////////////////////////////////////////////////////////////////////////// */
