/* /////////////////////////////////////////////////////////////////////////////
 * File:        reclspp_filesearch.h
 *
 * Purpose:     recls C++ mapping - FileSearch class.
 *
 * Created:     18th August 2003
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLSPP_FILESEARCH
#define RECLS_INCL_H_RECLSPP_FILESEARCH

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLSPP_FILESEARCH_MAJOR       2
# define RECLS_VER_H_RECLSPP_FILESEARCH_MINOR       0
# define RECLS_VER_H_RECLSPP_FILESEARCH_REVISION    2
# define RECLS_VER_H_RECLSPP_FILESEARCH_EDIT        14
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file reclspp_filesearch.h Definition of the \link recls::cpp::FileSearch FileSearch\endlink class, part of 
 * the \ref group_recls_cpp "recls C++ mapping"
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "reclspp.h"
#include "reclspp_search.h"
#include <recls_assert.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{

namespace cpp
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// This class provides functionality for executing a file search
///
/// \ingroup group_recls_cpp
class FileSearch
	: public Search
{
/// \name Types
public:
/// @{
	typedef FileSearch	class_type;
/// @}

/// \name Construction
/// @{
public:
    /// Creates a search for the items identified by \c pattern in the directory \c rootDir, according to the given \c flags.
    ///
    /// \param rootDir The starting directory for the search. If NULL, or the empty string, uses the current directory
    /// \param pattern The search pattern, e.g. "*.h". If NULL, or the empty string, uses "*.*" (Win32) or "*" (UNIX)
    /// \param flags A combination of enumerants from the \c RECLS_FLAG enumeration
    FileSearch(recls_char_t const *rootDir, recls_char_t const *pattern, recls_uint32_t flags);
    /// Destructor
    ~FileSearch();
/// @}

// Implementation
private:
    static ctor_args_type CreateSearch_(char const *rootDir, char const *pattern, recls_uint32_t flags)
	{
		hrecls_t    hSrch;
		recls_rc_t  rc = Recls_Search(rootDir, pattern, flags, &hSrch);

		return Search::ctor_args_type(hSrch, rc);
	}

// Not to be implemented
private:
    FileSearch(class_type const &rhs);
    class_type &operator =(class_type const &rhs);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

inline FileSearch::FileSearch(recls_char_t const *rootDir, recls_char_t const *pattern, recls_uint32_t flags)
    : Search(CreateSearch_(rootDir, pattern, flags))
{}

inline FileSearch::~FileSearch()
{}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace cpp */
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLSPP_FILESEARCH */

/* ////////////////////////////////////////////////////////////////////////// */
