/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_FileSearch.cpp
 *
 * Purpose:     JNI implementation file for the FileSearch class
 *
 * Created      21st November 2003
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include <recls_FileSearch.h>

#include <recls.h>
#if defined(RECLS_PLATFORM_IS_WIN32)
# include <recls_win32.h>
#elif defined(RECLS_PLATFORM_IS_UNIX)
# include <recls_unix.h>
#else
# error Platform not (yet) recognised
#endif /* platform */

#include <stlsoft.h>
#include <stlsoft_auto_buffer.h>
#include <stlsoft_malloc_allocator.h>

#include <stdarg.h>

#include "recls_jni_util.h"

/* ////////////////////////////////////////////////////////////////////////// */

#if !defined(RECLS_NO_NAMESPACE)
using recls::hrecls_t;

using recls::Recls_Search;
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/*
 * Class:     org_recls_FileSearch
 * Method:    initialise
 * Signature: (Ljava/lang/String;Ljava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_org_recls_FileSearch_initialise
  (JNIEnv *henv, jobject obj, jstring jsSearchRoot, jstring jsPattern, jint flags)
{
    // Need to 
    recls_char_t const			*searchRoot		=   SafeStringUTFChars(henv, jsSearchRoot, NULL);
    recls_char_t const			*pattern		=   SafeStringUTFChars(henv, jsPattern, NULL);

    static const recls_char_t	searchRoot_[2]	=	{ '.', '\0' };
#if defined(RECLS_PLATFORM_IS_WIN32)
    static const recls_char_t	pattern_[4]		=	{ '*', '.', '*', '\0' };
#elif defined(RECLS_PLATFORM_IS_UNIX)
    static const recls_char_t	pattern_[2]		=	{ '*', '\0' };
#else
# error Platform not (yet) recognised
#endif /* platform */

    if(0 == *searchRoot)
    {
        searchRoot = searchRoot_;
    }
    if(0 == *pattern)
    {
        pattern = pattern_;
    }

    hrecls_t    hSrch;
    recls_rc_t  rc  =   Recls_Search(searchRoot, pattern, flags, &hSrch);

    if( RECLS_FAILED(rc) &&
        rc != RECLS_RC_NO_MORE_DATA)
    {
        ThrowReclsException(henv, "Search failed", rc);
    }

    return reinterpret_cast<jint>(hSrch);
}

/* ////////////////////////////////////////////////////////////////////////// */
