/* /////////////////////////////////////////////////////////////////////////////
 * File:        reclstl.h
 *
 * Purpose:     recls STL mapping.
 *
 * Created:     10th September 2003
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLSTL
#define RECLS_INCL_H_RECLSTL

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLSTL_MAJOR      2
# define RECLS_VER_H_RECLSTL_MINOR      0
# define RECLS_VER_H_RECLSTL_REVISION   2
# define RECLS_VER_H_RECLSTL_EDIT       11
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"

#if RECLS_VER < RECLS_VER_1_1_1
# error The recls STL mapping requires version 1.1.1 or later of the recls C API
#endif /* RECLS_VER < 1.1.1 */

#ifdef RECLS_STL_STRING_HEADER
# include RECLS_STL_STRING_HEADER
#else
# include <string>
#endif /* RECLS_STL_STRING_HEADER */

// FTP?
#if defined(RECLS_PLATFORM_IS_WIN32)
# if !defined(RECLS_INCL_H_RECLS_FTP)
#  include "recls_ftp.h"
# endif /* !RECLS_INCL_H_RECLS_FTP */
#endif /* RECLS_PLATFORM_IS_WIN32 */

/** \file reclstl.h The root header for the \ref group_recls_stl "recls STL mapping" */

#include <stlsoft.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{

namespace stl
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

#ifdef RECLS_DOCUMENTATION_SKIP_SECTION

/// \brief string_t The string type to be used in the \c reclstl (\c recls::stl) namespace
///
/// \ingroup group_recls_stl
/// 
/// \note By default \c string_t is defined to be <code>std::string</code>, but this can be
/// overridden by specifying \c RECLS_STL_STRING_HEADER, e.g. <code>"mystring.h"</code>,
/// <b>and</b> also defining \c RECLS_STL_STRING_T, e.g. <code>"my_string"</code> which will
/// be typedef'd to \c string_t
typedef std::string             string_t;

#else /* ? RECLS_DOCUMENTATION_SKIP_SECTION */

# ifdef RECLS_STL_STRING_HEADER
#  ifndef RECLS_STL_STRING_T
#   error RECLS_STL_STRING_T must be defined when RECLS_STL_STRING_HEADER is defined
#  endif /* !RECLS_STRING_T */
typedef RECLS_STL_STRING_T			string_t;
# else /* ? RECLS_STRING_HEADER */
typedef stlsoft_ns_qual_std(string)	string_t;
# endif /* RECLS_STRING_HEADER */
#endif /* RECLS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace stl */
} /* namespace recls */
namespace reclstl = ::recls::stl;
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLSTL */

/* ////////////////////////////////////////////////////////////////////////// */
