/* /////////////////////////////////////////////////////////////////////////////
 * File:        txtcnv.cpp
 *
 * Purpose:     Implementation file for the txtcnv project.
 *
 * Created:     19th March 2004
 * Updated:     10th January 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 2004-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

#if defined(__DMC__)
# include <iostream.h>
#else /* ? __DMC__ */
# include <iostream>
# include <fstream>

using std::cout;
using std::cerr;
using std::endl;
#endif /* __DMC__ */

#include <recls.h>
#include <recls_assert.h>
#include <reclstl_search_sequence.h>

#include <stlsoft.h>
#include <stlsoft_string_access.h>
#include <stlsoft_string_tokeniser.h>

#include <string>
#include <vector>

/* /////////////////////////////////////////////////////////////////////////////
 * Version information
 *
 * Normally, Synesis Software components & tools are "versioned" by making use
 * of tool-generated header files, one of which - MBldHdr.h - defines symbolic
 * constants representing major, minor, revision and build numbers.
 *
 *
 * Since this file is to be available in a more public forum, these constants 
 * are merely defined here, rather than encumber any developers who may wish to
 * compile it with a lot of Synesis-specific gunk.
 */

#ifdef _SYNSOFT_INTERNAL_BUILD
# include "MBldHdr.h"

 const int  _mccVerHi   =   __SYV_MAJOR;
 const int  _mccVerLo   =   __SYV_MINOR;
 const int  _mccVerRev  =   __SYV_REVISION;
 const int  _mccBldNum  =   __SYV_BUILDNUMBER;

#else
 const int  _mccVerHi   =   1;
 const int  _mccVerLo   =   0;
 const int  _mccVerRev  =   1;
 const int  _mccBldNum  =   1;
#endif /* _SYNSOFT_INTERNAL_BUILD */

#ifdef _SYNSOFT_INTERNAL_BUILD
# include "MLCompId.h"
#else
# define _nameSynesisSoftware               "Synesis Software (Pty) Ltd"
# define _wwwSynesisSoftware_SystemTools    "http://synesis.com.au/"
# define _emailSynesisSoftware_SystemTools  "software@synesis.com.au"
#endif /* _SYNSOFT_INTERNAL_BUILD */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

enum TextType
{
        unknown
    ,   fmtWindows
    ,   fmtUNIX
};

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit);

/* ////////////////////////////////////////////////////////////////////////// */

int main(int argc, char *argv[])
{
#if 0
    ::Sleep(100000);
#endif /* 0 */

    int                     iRet        =   0;
    int                     i;
    bool                    bVerbose    =   true;
    bool                    bAfter      =   true;
    char const              *rootDir    =   NULL;
    char const              *pattern    =   NULL;
    recls::recls_uint32_t   flags       =   recls::RECLS_F_FILES | recls::RECLS_F_RECURSIVE;
    TextType                fmt         =   unknown;

    cout << "txtcnv: " << __STLSOFT_COMPILER_LABEL_STRING << endl;

    for(i = 1; i < argc; ++i)
    {
        const char  *arg    =   argv[i];

        if(arg[0] == '-')
        {
            if(arg[1] == '-')
            {
                /* -- arguments */
                usage(true);
            }
            else
            {
                /* - arguments */
                switch(arg[0])
                {
                    case    'a':
                        bAfter = true;
                        break;
                    case    'b':
                        bAfter = false;
                        break;
                    case    'r':
                        break;
                    case    'R':
                        flags &= ~(recls::RECLS_F_RECURSIVE);
                        break;
                    case    's':
                        bVerbose = false;
                        break;
                    case    'v':
                        bVerbose = true;
                        break;
                    case    'U':
                        if(fmt != unknown)
                        {
                            usage(true);
                        }
                        fmt = fmtUNIX;
                        break;
                    case    'W':
                        if(fmt != unknown)
                        {
                            usage(true);
                        }
                        fmt = fmtWindows;
                        break;
                }
            }
        }
        else
        {
            /* other arguments */
            if(NULL == rootDir)
            {
                rootDir = arg;
            }
            else if(NULL == pattern)
            {
                pattern = arg;
            }
            else
            {
                usage(true);
            }
        }
    }

    if(NULL == rootDir)
    {
        rootDir = ".";
    }

    if(NULL == pattern)
    {
        pattern = recls::Recls_GetWildcardsAll();
    }

    typedef stlsoft::string_tokeniser<std::string, char>  tokeniser_t;

    tokeniser_t     tokens(pattern, ';');
    
    for(tokeniser_t::const_iterator begin = tokens.begin(), end = tokens.end(); begin != end; ++begin)
    {
        typedef reclstl::basic_search_sequence<char>    sequence_t;

        sequence_t  search(rootDir, stlsoft::c_str_ptr(*begin), flags);

        for(sequence_t::const_iterator begin = search.begin(), end = search.end(); begin != end; ++begin)
        {
//          std::vector<stlsoft::byte_t>    contents;
//          std::ifstream                   in(stlsoft::c_str_ptr((*begin)), std::ios::in | std::ios::binary);
//
//          readall(in, contents);
//          in.close();
//          if(modify(contents, fmt))
//          {
//              std::ifstream(
//              std::ifstream::pos_type len =   in.seekg(0, std::ios_base::end).tellg();
//
//
//              in.seekg(0, std::ios_base::beg);
//              ;
//          }

        }
    }

    return iRet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * usage
 */

void usage(int bExit)
{
    fprintf(stderr, "\n");
    fprintf(stderr, "  Synesis Software Text File Conversion Utility v%d.%d.%d.%04d\n\n", _mccVerHi, _mccVerLo, _mccVerRev, _mccBldNum);
    fprintf(stderr, "  USAGE: txtcnv { -a | -b } { -W | -U } [{ -r | -R | -v | -s }] <root> <search-spec>\n");
    fprintf(stderr, "    where:\n\n");
    fprintf(stderr, "    -a             - Take copy of converted file, and replace original after conversion. (Default)\n");
    fprintf(stderr, "    -b             - Take copy of file, and convert into original.\n");
    fprintf(stderr, "    -W             - Windows format\n");
    fprintf(stderr, "    -U             - UNIX format\n");
    fprintf(stderr, "    -r             - recursive search. (Default)\n");
    fprintf(stderr, "    -R             - suppresses recursive search.\n");
    fprintf(stderr, "    -v             - verbose output. Prints out the changes. (Default)\n");
    fprintf(stderr, "    -s             - succinct output. Prints path only.\n");
    fprintf(stderr, "    <root>         - root directory for the processing.\n");
    fprintf(stderr, "    <search-spec>  - one or more file search specifications, separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       \"*.exe\"\n");
    fprintf(stderr, "                       \"myfile.ext\"\n");
    fprintf(stderr, "                       \"*.exe;*.dll\"\n");
    fprintf(stderr, "                       \"*.xl?;report.*\"\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "  Contact %s\n", _nameSynesisSoftware);
    fprintf(stderr, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
    fprintf(stderr, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stderr, "\n");

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

/* ////////////////////////////////////////////////////////////////////////// */
