/* /////////////////////////////////////////////////////////////////////////////
 * File:        ReclsFtpSearchDirectoryNode_win32.h
 *
 * Purpose:     ReclsFtpSearchDirectoryNode class, for Win32.
 *
 * Created:     31st May 2004
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"

#include <stlsoft.h>
#include <stlsoft_simple_string.h>

#include <inetstl.h>
#include <inetstl_filesystem_traits.h>
#include <inetstl_findfile_sequence.h>
#ifdef USE_SEARCHSPEC_SEQ
# include <inetstl_searchspec_sequence.h>
#endif /* USE_SEARCHSPEC_SEQ */

// The Dinkumware list implementation causes Visual C++ 7.1 to give C4702 warning, so we don't use it
#if defined(RECLS_COMPILER_IS_MSVC) && \
    _MSC_VER >= 1310
# include <vector>
#else /* ? compiler */
# include <list>
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

#ifdef USE_SEARCHSPEC_SEQ
inetstl_ns_using(searchspec_sequence)
#endif /* USE_SEARCHSPEC_SEQ */
inetstl_ns_using(basic_findfile_sequence)

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

class ReclsFtpSearchDirectoryNode
//    : public ReclsDNode
{
public:
    typedef recls_char_t                                        char_type;
    typedef ReclsFtpSearchDirectoryNode                         class_type;
    typedef inetstl_ns_qual(filesystem_traits)<recls_char_t>    traits_type;
private:
    typedef stlsoft_ns_qual(basic_simple_string)<recls_char_t>  string_type;
    typedef basic_findfile_sequence<char_type, traits_type>     file_find_sequence_type;
#if defined(RECLS_COMPILER_IS_MSVC) && \
    _MSC_VER >= 1310
    typedef stlsoft_ns_qual_std(vector)<string_type>            directory_sequence_type;
#else /* ? compiler */
    typedef stlsoft_ns_qual_std(list)<string_type>              directory_sequence_type;
#endif /* compiler */
#ifdef USE_SEARCHSPEC_SEQ
    typedef searchspec_sequence<file_find_sequence_type>        entry_sequence_type;
#else /* ? USE_SEARCHSPEC_SEQ */
    typedef file_find_sequence_type                             entry_sequence_type;
#endif /* USE_SEARCHSPEC_SEQ */

// Construction
#if defined(RECLS_COMPILER_IS_GCC)
protected:
#else /* ? compiler */
private:
#endif /* compiler */
    ReclsFtpSearchDirectoryNode(HINTERNET connection, recls_uint32_t flags, char_type const *rootDir, char_type const *pattern);
public:
    virtual ~ReclsFtpSearchDirectoryNode();

    static ReclsFtpSearchDirectoryNode *FindAndCreate(HINTERNET connection, recls_uint32_t flags, char_type const *rootDir, char_type const *pattern);
    static ReclsFtpSearchDirectoryNode *FindAndCreate(HINTERNET connection, recls_uint32_t flags, char_type const *rootDir, char_type const *subDir, char_type const *pattern);

// ReclsDNode methods
public:
    /* virtual */ recls_rc_t GetNext();
    /* virtual */ recls_rc_t GetDetails(recls_info_t *pinfo);
    /* virtual */ recls_rc_t GetNextDetails(recls_info_t *pinfo);

// Implementation
private:
    recls_rc_t      Initialise();

    recls_bool_t    is_valid_() const;

#if defined(RECLS_COMPILER_IS_BORLAND)
    static directory_sequence_type::const_iterator _select_iter(int b, directory_sequence_type::const_iterator trueVal, directory_sequence_type::const_iterator falseVal)
    {
        // I can't explain it, but Borland does not like the tertiary operator and the copy-ctors of the iterators
        if(b)
        {
            return trueVal;
        }
        else
        {
            return falseVal;
        }
    }
#endif /* !RECLS_COMPILER_IS_BORLAND */
    static int _ssFlags_from_reclsFlags(recls_uint32_t flags)
    {
#if 0
        recls_assert(0 == (flags & RECLS_F_LINKS));     // Doesn't work with links
        recls_assert(0 == (flags & RECLS_F_DEVICES));   // Doesn't work with devices
#endif /* 0 */

        int ssFlags = 0;

        if(0 != (flags & RECLS_F_FILES))
        {
#ifdef USE_SEARCHSPEC_SEQ
            // Because Digital Mars 8.40- has a problem, we must access the typedef separately from the enum value
            typedef entry_sequence_type::find_sequence_type sequence_t;

            ssFlags |= sequence_t::files;

//            ssFlags |= entry_sequence_type::find_sequence_type::files;
#else /* ? USE_SEARCHSPEC_SEQ */
            ssFlags |= entry_sequence_type::files;
#endif /* USE_SEARCHSPEC_SEQ */
        }
        if(0 != (flags & RECLS_F_DIRECTORIES))
        {
#ifdef USE_SEARCHSPEC_SEQ
            // Because Digital Mars 8.40- has a problem, we must access the typedef separately from the enum value
            typedef entry_sequence_type::find_sequence_type sequence_t;

            ssFlags |= sequence_t::directories;

//            ssFlags |= entry_sequence_type::find_sequence_type::directories;
#else /* ? USE_SEARCHSPEC_SEQ */
            ssFlags |= entry_sequence_type::directories;
#endif /* USE_SEARCHSPEC_SEQ */
        }

        return ssFlags;
    }

// Members
private:
    HINTERNET                               m_connection;
    recls_info_t                            m_current;
    ReclsFtpSearchDirectoryNode             *m_dnode;
    recls_uint32_t const                    m_flags;
    entry_sequence_type                     m_entries;
    entry_sequence_type::const_iterator     m_entriesBegin;
    directory_sequence_type                 m_directories;
    directory_sequence_type::const_iterator m_directoriesBegin;
    string_type                             m_rootDir;
    string_type                             m_pattern;

// Not to be implemented
private:
    ReclsFtpSearchDirectoryNode(class_type const &);
    class_type &operator =(class_type const &);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
