/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_internal.cpp
 *
 * Purpose:     Implementation file for the recls API internal helpers.
 *
 * Created:     16th August 2003
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"
#include "recls_internal.h"
#include "recls_assert.h"
#include "recls_util.h"

#include <stlsoft.h>
#if !defined(RECLS_COMPILER_IS_VECTORC)
# include <stlsoft_nulldef.h>
#endif /* compiler */

#include <string.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#if defined(RECLS_CHAR_TYPE_IS_CHAR)
# define recls_strncpy__    strncpy
# define recls_strlen__     strlen
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
# define recls_strncpy__    wcsncpy
# define recls_strlen__     wcslen
#else
# error 
#endif /* RECLS_CHAR_TYPE_IS_???? */

size_t recls_strncpy(recls_char_t *dest, size_t cchDest, recls_char_t const *src, size_t cchSrc)
{
    size_t  cchWritten;

    if(cchDest < cchSrc)
    {
        /* Just to straight strncpy. */
        recls_strncpy__(dest, src, cchDest);

        cchWritten = cchDest;
    }
    else
    {
        recls_strncpy__(dest, src, cchSrc);

        if(cchSrc < cchDest)
        {
            /* Fill the rest up with blanks. */

            memset(&dest[cchSrc], 0, sizeof(recls_char_t) * (cchDest - cchSrc));
        }

        cchWritten = cchSrc;
    }

    return cchWritten;
}

size_t recls_strlen(recls_char_t const *s)
{
    return recls_strlen__(s);
}

/* ////////////////////////////////////////////////////////////////////////// */

#if defined(RECLS_COMPILER_IS_DMC) || \
    defined(RECLS_COMPILER_IS_WATCOM)
RECLS_FNDECL(size_t) Recls_GetStringProperty_(  struct recls_strptrs_t const *      ptrs
                                            ,   recls_char_t *                      buffer
                                            ,   size_t                              cchBuffer)
#else
RECLS_FNDECL(size_t) Recls_GetStringProperty_(  struct recls_strptrs_t const *const ptrs
                                            ,   recls_char_t *const                 buffer
                                            ,   size_t const                        cchBuffer)
#endif /* RECLS_COMPILER_IS_DMC || RECLS_COMPILER_IS_WATCOM */
{
    recls_assert(NULL != ptrs);

    size_t  cch =   ptrs->end - ptrs->begin;

    if(NULL != buffer)
    {
        cch = recls_strncpy(buffer, cchBuffer, ptrs->begin, cch);
    }

    return cch;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
