/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_roots_win32.cpp
 *
 * Purpose:     Emumeration of the roots for the Win32 platform.
 *
 * Created:     20th March 2004
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"
#include "recls_internal.h"
#include "recls_assert.h"
#include "recls_util.h"

#include <stlsoft.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

static BOOL is_drive_present(char letter)
{
    DWORD   dwFlags =   0;
    DWORD   dwDummy =   0;
    char    root[]  =   "?:\\\0";

    root[0] = letter;

    return ::GetVolumeInformationA(root, NULL, 0, NULL, &dwDummy, &dwFlags, NULL, 0);
}

#if defined(RECLS_COMPILER_IS_WATCOM)
static size_t check_drives(char *drives)
#else /* ? compiler */
static size_t check_drives(char (*drives)[26])
#endif /* compiler */
{
    size_t n = 0;

    for(size_t i = 0; i < 26; ++i)
    {
        char    letter = static_cast<char>('A' + i);

        if(is_drive_present(letter))
        {
#if defined(RECLS_COMPILER_IS_WATCOM)
            drives[n] = letter;
#else /* ? compiler */
            (*drives)[n] = letter;
#endif /* compiler */

            ++n;
        }
    }

    return n;
}

RECLS_LINKAGE_C size_t Recls_GetRoots(  recls_root_t    *roots
                                    ,   size_t          cRoots)
{
    recls_assert(NULL == roots || 0 != cRoots);

    char    drives[26];
#if defined(RECLS_COMPILER_IS_WATCOM)
    size_t  n   =   check_drives(&drives[0]);
#else /* ? compiler */
    size_t  n   =   check_drives(&drives);
#endif /* compiler */

    if(NULL != roots)
    {
        if(n < cRoots)
        {
            cRoots = n;
        }

        for(size_t i = 0, n = 0; i < cRoots; ++i, ++n)
        {
            roots[i].name[0] = drives[i];
            roots[i].name[1] = ':';
            roots[i].name[2] = '\\';
            roots[i].name[3] = '\0';
        }
    }

    return n;
}

RECLS_LINKAGE_C recls_char_t const *Recls_GetPathNameSeparator(void)
{
    static recls_char_t const   s_pathNameSeparator[] = { '\\', '\0' };

    return s_pathNameSeparator;
}

RECLS_LINKAGE_C recls_char_t const *Recls_GetPathSeparator(void)
{
    static recls_char_t const   s_pathSeparator[] = { ';', '\0' };

    return s_pathSeparator;
}

RECLS_LINKAGE_C recls_char_t const *Recls_GetWildcardsAll(void)
{
    static recls_char_t const   s_wildcardsAll[] = { '*', '.', '*', '\0' };

    return s_wildcardsAll;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
