/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_wininet_dl.h
 *
 * Purpose:     Delay-loading functionality for the recls library.
 *
 * Created:     9th September 2004
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_WININET_DL_MAJOR     1
# define RECLS_VER_H_RECLS_WININET_DL_MINOR     0
# define RECLS_VER_H_RECLS_WININET_DL_REVISION  2
# define RECLS_VER_H_RECLS_WININET_DL_EDIT      2
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_wininet_dl.h Delay-loading functionality for the recls library */

#if !defined(RECLS_DELAY_LOAD_WININET)
# error Cannot include recls_wininet_dl.h unless RECLS_DELAY_LOAD_WININET is defined
#endif /* RECLS_DELAY_LOAD_WININET */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"
#include <wininet.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Macros, definitions and typedefs
 */

#ifndef INTERNETAPI_
# define INTERNETAPI_(type)     EXTERN_C type STDAPICALLTYPE
#endif /* !INTERNETAPI_ */

#ifndef BOOLAPI
# define BOOLAPI                INTERNETAPI_(BOOL)
#endif /* !BOOLAPI */

#if defined(RECLS_COMPILER_IS_DMC)
typedef unsigned long           DWORD_PTR_;
# define DWORD_PTR              DWORD_PTR_
#endif /* RECLS_COMPILER_IS_DMC */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

BOOLAPI recls_dl_FtpGetCurrentDirectoryA(
    IN HINTERNET hConnect,
    OUT LPSTR lpszCurrentDirectory,
    IN OUT LPDWORD lpdwCurrentDirectory
    );
BOOLAPI recls_dl_FtpGetCurrentDirectoryW(
    IN HINTERNET hConnect,
    OUT LPWSTR lpszCurrentDirectory,
    IN OUT LPDWORD lpdwCurrentDirectory
    );

INTERNETAPI_(HINTERNET) recls_dl_InternetOpenA(
    IN LPCSTR lpszAgent,
    IN DWORD dwAccessType,
    IN LPCSTR lpszProxy OPTIONAL,
    IN LPCSTR lpszProxyBypass OPTIONAL,
    IN DWORD dwFlags
    );
INTERNETAPI_(HINTERNET) recls_dl_InternetOpenW(
    IN LPCWSTR lpszAgent,
    IN DWORD dwAccessType,
    IN LPCWSTR lpszProxy OPTIONAL,
    IN LPCWSTR lpszProxyBypass OPTIONAL,
    IN DWORD dwFlags
    );

INTERNETAPI_(HINTERNET) recls_dl_InternetConnectA(
    IN HINTERNET hInternet,
    IN LPCSTR lpszServerName,
    IN INTERNET_PORT nServerPort,
    IN LPCSTR lpszUserName OPTIONAL,
    IN LPCSTR lpszPassword OPTIONAL,
    IN DWORD dwService,
    IN DWORD dwFlags,
    IN DWORD_PTR dwContext
    );
INTERNETAPI_(HINTERNET) recls_dl_InternetConnectW(
    IN HINTERNET hInternet,
    IN LPCWSTR lpszServerName,
    IN INTERNET_PORT nServerPort,
    IN LPCWSTR lpszUserName OPTIONAL,
    IN LPCWSTR lpszPassword OPTIONAL,
    IN DWORD dwService,
    IN DWORD dwFlags,
    IN DWORD_PTR dwContext
    );

BOOLAPI recls_dl_InternetCloseHandle(
    IN HINTERNET hInternet
    );

BOOLAPI recls_dl_InternetFindNextFileA(
    IN HINTERNET hFind,
    OUT LPVOID lpvFindData
    );
BOOLAPI recls_dl_InternetFindNextFileW(
    IN HINTERNET hFind,
    OUT LPVOID lpvFindData
    );

INTERNETAPI_(HINTERNET) recls_dl_FtpFindFirstFileA(
    IN HINTERNET hConnect,
    IN LPCSTR lpszSearchFile OPTIONAL,
    OUT LPWIN32_FIND_DATAA lpFindFileData OPTIONAL,
    IN DWORD dwFlags,
    IN DWORD_PTR dwContext
    );
INTERNETAPI_(HINTERNET) recls_dl_FtpFindFirstFileW(
    IN HINTERNET hConnect,
    IN LPCWSTR lpszSearchFile OPTIONAL,
    OUT LPWIN32_FIND_DATAW lpFindFileData OPTIONAL,
    IN DWORD dwFlags,
    IN DWORD_PTR dwContext
    );

#define FtpGetCurrentDirectoryA recls_dl_FtpGetCurrentDirectoryA
#define FtpGetCurrentDirectoryW recls_dl_FtpGetCurrentDirectoryW
#define InternetOpenA           recls_dl_InternetOpenA
#define InternetOpenW           recls_dl_InternetOpenW
#define InternetConnectA        recls_dl_InternetConnectA
#define InternetConnectW        recls_dl_InternetConnectW
#define InternetFindNextFileA   recls_dl_InternetFindNextFileA
#define InternetFindNextFileW   recls_dl_InternetFindNextFileW
#define FtpFindFirstFileA       recls_dl_FtpFindFirstFileA
#define FtpFindFirstFileW       recls_dl_FtpFindFirstFileW
#define InternetCloseHandle     recls_dl_InternetCloseHandle

/* ////////////////////////////////////////////////////////////////////////// */
