/* /////////////////////////////////////////////////////////////////////////////
 * File:        C.c
 *
 * Purpose:     Implementation file for the C project.
 *
 * Created:     15th August 2003
 * Updated:     6th March 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 2003-2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 */

/** \file C.c Test program for the \ref group_recls "recls C API" */

/* Remove this definition to use platform-specific aspects of the API */
#ifdef _DEBUG
#else /* ? _DEBUG */
# define RECLS_PURE_API
#endif /* _DEBUG */

/* Open-RJ Header Files */
#include <recls.h>
#include <recls_assert.h>
#include <recls_minmax.h>

#if RECLS_VER < RECLS_VER_1_5_3
# error This file now requires version 1.5.3 or later of the recls C API
#endif /* RECLS_VER < 1.5.3 */

/* Standard C Library Files */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if defined(__GNUC__)
# ifdef alloca
#  undef alloca
# endif /* !alloca */
# define alloca				__builtin_alloca
#elif defined(WIN32)
# include <malloc.h>
# ifndef alloca
#  define alloca            _alloca
# endif /* !alloca */
#else /* ? OS */
# include <alloca.h>
#endif /* OS */


#ifdef RECLS_PLATFORM_IS_UNIX
# include <unistd.h>
#endif /* RECLS_PLATFORM_IS_UNIX */

#ifdef _MSC_VER
# include <crtdbg.h>
#endif /* _MSC_VER */

/* /////////////////////////////////////////////////////////////////////////////
 * Compiler compatibility
 */

#if defined(RECLS_COMPILER_IS_MSVC)
# if _MSC_VER < 1300
#  pragma warning(disable : 4127)
# endif /* _MSC_VER < 1300 */
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 */

#ifndef NUM_ELEMENTS
# ifdef __DMC__
# define NUM_ELEMENTS(x)        (sizeof(x) / sizeof((x)[0]))
# else /* ? compiler */
# define NUM_ELEMENTS(x)        (sizeof(x) / sizeof(0[(x)]))
# endif /* compiler */
#endif /* !NUM_ELEMENTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */

static size_t path_max(void)
{
#if defined(WIN32)
	return 1 + _MAX_PATH;
#elif defined(PATH_MAX)
	return 1 + PATH_MAX;
#else /* ? PATH_MAX */
	return 1 + 1 + pathconf("/", _PC_PATH_MAX);
#endif /* PATH_MAX */
}

#define alloca_path()	((recls_char_t*)alloca(path_max()))

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason);
static int processDirectory(char const *rootDir, char const *pattern, long flags, int bSuccinct);

/* /////////////////////////////////////////////////////////////////////////////
 * Main
 */

/** Program main */
int main(int argc, char *argv[])
{
#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemState    memState;
#endif /* _MSC_VER && _MSC_VER */
    int             iRet            =   0;
    int             i;
    int             totalFound      =   0;
    int             bAllHardDrives  =   0;
    char const      *host	        =   NULL;
    char const      *username       =   NULL;
    char const      *password       =   NULL;
    char const      *pattern        =   NULL;
    char const      *rootDir        =   NULL;
    recls_uint32_t  flags           =   RECLS_F_RECURSIVE;
    int             bSuccinct       =   0;

    for(i = 1; i < argc; ++i)
    {
        const char  *arg    =   argv[i];

        if(arg[0] == '-')
        {
            if(arg[1] == '-')
            {
                /* -- arguments */
            }
            else
            {
                /* - arguments */
                switch(arg[1])
                {
                    case    '?':
                        usage(1, NULL);
                        break;
                    case    'R':    /* Do not recurse */
                        flags &= ~(RECLS_F_RECURSIVE);
                        break;
                    case    'p':    /* Show directory parts */
                        flags |= RECLS_F_DIRECTORY_PARTS;
                        break;
                    case    'f':    /* Find files */
                        flags |= RECLS_F_FILES;
                        break;
                    case    'd':    /* Find directories */
                        flags |= RECLS_F_DIRECTORIES;
                        break;
                    case    'h':    /* Searches from hard drives */
                        bAllHardDrives = 1;
                        break;
                    case    's':    /* Show only the full path; WHEREIS functionality */
                        bSuccinct = 1;
                        break;
                    case    'H':    /* FTP host */
                        host = arg + 2;
                        break;
                    case    'U':    /* FTP username */
                        username = arg + 2;
                        break;
                    case    'P':    /* FTP password */
                        password = arg + 2;
                        break;
                    default:
                        usage(1, "Invalid argument(s) specified");
                        break;
                }
            }
        }
        else
        {
            /* other arguments */
            if(NULL == pattern)
            {
                pattern = arg;
            }
            else if(NULL == rootDir)
            {
                rootDir = arg;
            }
            else
            {
                usage(1, "Invalid argument(s) specified");
            }
        }
    }

	if( NULL != password &&
		(	NULL == host ||
			NULL == username))
	{
		usage(1, "Must specify host and username if specifying password");
	}

	if( NULL != username &&
		NULL == host)
	{
		usage(1, "Must specify host if specifying username");
	}

	if( NULL != host &&
		bAllHardDrives)
	{
		usage(1, "-h flag meaningless to FTP searches");
	}

    if( NULL != host &&
		NULL == rootDir)
    {
		usage(1, "Must specify a root directory for FTP searches");
    }

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemCheckpoint(&memState);
#endif /* _MSC_VER && _MSC_VER */

    /* Search for files if neither files or directories specified/
     *
     * Even though this is not necessary, because the recls API provides the
     * same interpretation, it's best to be explicit.
     */
    if(0 == (flags & (RECLS_F_FILES | RECLS_F_DIRECTORIES)))
    {
        flags |= RECLS_F_FILES;
    }

    if(NULL == pattern)
    {
        pattern = Recls_GetWildcardsAll();
    }

    if(NULL == rootDir)
    {
        rootDir = ".";
    }

	/* Check the size of rootDir */

	{
		char  const     *filePath	=   "..";
		char const		*fslash		=	strrchr(filePath, '/');
	#ifdef WIN32
		char const		*bslash		=	strrchr(filePath, '\\');
		char const		*slash		=	(NULL == fslash) ? bslash : (NULL == bslash) ? fslash : recls_max(bslash, fslash);
	#else /* ? WIN32 */
		char const		*slash		=	fslash;
	#endif /* WIN32 */
		size_t			cchDir		=	(NULL == slash) ? 0 : (size_t)(slash - filePath);
		char			*dir		=	(char*)alloca(1 + cchDir);

		recls_uint32_t  flags_       =   RECLS_F_FILES | RECLS_F_DIRECTORIES;
		hrecls_t        hSrch;
		recls_rc_t      rc;

		cchDir[strncpy(dir, filePath, 1 + cchDir)] = '\0';

		if(NULL != slash)
		{
			filePath += cchDir + 1;
		}

		rc = Recls_Search(dir, filePath, flags_, &hSrch);

		if(RECLS_SUCCEEDED(rc))
		{
			recls_info_t	entry;

			Recls_GetDetails(hSrch, &entry);

			Recls_CloseDetails(entry);

			Recls_SearchClose(hSrch);
		}
	}


#ifdef x_DEBUG
	{
		recls_bool_t		bEmpty	=	Recls_IsDirectoryEmpty(rootDir);

		recls_filesize_t	size	=	Recls_CalcDirectorySize(rootDir);


		((void)bEmpty);
		((void)size);
	}

	{
        recls_info_t    info;
		hrecls_t		hSrch;
		recls_rc_t		rc	=   Recls_Search(".", ".", RECLS_F_DIRECTORIES, &hSrch);

		Recls_GetDetails(hSrch, &info);

		Recls_IsDirectoryEntryEmpty(info);

		Recls_CloseDetails(info);
		Recls_SearchClose(hSrch);
	}

#endif /* _DEBUG */

    /* Initiate the search. */
    if(bAllHardDrives)
    {
        recls_root_t    roots[26];
        size_t          cRoots  =   Recls_GetRoots(roots, NUM_ELEMENTS(roots));

        for(i = 0; i < (int)cRoots; ++i)
        {
            totalFound += processDirectory(roots[i].name, pattern, flags, bSuccinct);
        }
    }
    else
    {
        totalFound = processDirectory(rootDir, pattern, flags, bSuccinct);
    }

    fprintf(stdout, "Total found: %d\n", totalFound);

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemDumpAllObjectsSince(&memState);
#endif /* _MSC_VER && _MSC_VER */

    return iRet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Function implementations
 */

/** Displays program usage */
void usage(int bExit, char const *reason)
{
    fprintf(stderr, "recls C Test Program\n\n");

	if(NULL != reason)
	{
		fprintf(stderr, "  Error: %s\n\n", reason);
	}

    fprintf(stderr, "Usage: C++ [-d] [-f] [-h] [-p] [-R] [-s] [-H<host>] [-U<username>] [-P<password>] [<pattern>] [<root-dir>]\n");
    fprintf(stderr, "\t-R               -   does not recurse; recursive search is the default\n");
    fprintf(stderr, "\t-p               -   evaluate and display directory parts\n");
    fprintf(stderr, "\t-d               -   search for directories\n");
    fprintf(stderr, "\t-f               -   search for files (this is the default, if -d is not specified)\n");
    fprintf(stderr, "\t-h               -   searches from hard drive roots. (Ignore <root-dir>.)\n");
    fprintf(stderr, "\t-s               -   succinct; shows only the path\n");
	fprintf(stderr, "\t-H<host>         -   performs an FTP search on the given host\n");
	fprintf(stderr, "\t-U<username>     -   username to use to log on to <host>\n");
	fprintf(stderr, "\t-P<password>     -   password to use to log on to <host>\n");
    fprintf(stderr, "\t<pattern>        -   search pattern, e.g. \"*.cpp\" (default is to search for all files)\n");
    fprintf(stderr, "\t<root-dir>       -   root directory of search; default is current working directory\n");

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

static int processDirectory(char const *rootDir, char const *pattern, long flags, int bSuccinct)
{
    hrecls_t    hSrch;
    recls_rc_t  rc          =   Recls_Search(rootDir, pattern, flags, &hSrch);
    int         totalFound  =   0;

    if(RECLS_FAILED(rc))
    {
        recls_char_t    err[100];

        Recls_GetErrorString(rc, err, 100);

        fprintf(stderr, "Failed to start search, with pattern \"%s\"; recls error: %s\n", pattern, err);
    }
    else
    {
        /* Iterate through the items, until done */
        recls_info_t    info;

        rc = Recls_GetDetails(hSrch, &info);

        for(; RECLS_SUCCEEDED(rc); rc = Recls_GetNextDetails(hSrch, &info))
        {
            int             i;
            int             off;
            int             extLen;
            int             cDirParts;
            recls_char_t    *path		=	alloca_path();
#ifdef RECLS_PLATFORM_API_WIN32
            recls_char_t    drive;
#endif /* RECLS_PLATFORM_API_WIN32 */
            recls_char_t    *dir		=	alloca_path();
            recls_char_t    *dirPath	=	alloca_path();
            recls_char_t    *file		=	alloca_path();
            recls_char_t    *fileName	=	alloca_path();
            recls_char_t    *fileExt	=	alloca_path();

            recls_char_t    *pathCheck	=	alloca_path();
            recls_char_t    *dirCheck	=	alloca_path();
            recls_char_t    *fileCheck	=	alloca_path();

            ++totalFound;

            pathCheck[0] = '\0';
            dirCheck[0] = '\0';
            fileCheck[0] = '\0';

#ifdef x_DEBUG
            {
                recls_uint32_t  cBlocks;
                Recls_OutstandingDetails(hSrch, &cBlocks);
                printf("\n%d outstanding blocks\n", cBlocks);
            }
#endif /* _DEBUG */

            Recls_GetPathProperty(info, path, path_max());
            printf("  %s\n", path);
            if(!bSuccinct)
            {
                Recls_GetDirectoryPathProperty(info, dirPath, path_max());
                printf("  %s\n", dirPath);
#ifdef RECLS_PLATFORM_API_WIN32
                Recls_GetDriveProperty(info, &drive);
                if(isupper(path[0]))
                {
                    drive = (recls_char_t)toupper(drive);
                }
                else
                {
                    drive = (recls_char_t)tolower(drive);
                }
                printf("  %c\n", drive);
                sprintf(pathCheck, "%c:", drive);
                off = 2;
#elif defined(RECLS_PLATFORM_IS_WIN32)
                off = 2;
#else /* platform */
                off = 0;
#endif /* RECLS_PLATFORM_API_WIN32 */
                Recls_GetDirectoryProperty(info, dir, path_max());
                printf("  %*s\n", off + (int)strlen(dir), dir);

                if((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS)
                {
                    for(i = 0, cDirParts = Recls_GetDirectoryPartProperty(info, -1, NULL, 0); i < cDirParts; ++i)
                    {
                        recls_char_t    *dirPart	=	alloca_path();

                        off += Recls_GetDirectoryPartProperty(info, i, dirPart, path_max());
                        printf("  %*s\n", off, dirPart);

                        strcat(pathCheck, dirPart);
                        strcat(dirCheck, dirPart);
                    }
                }
                else
                {
                    off += strlen(dir);
                    strcat(pathCheck, dir);
                }

                off +=   Recls_GetFileProperty(info, file, path_max());
                         Recls_GetFileNameProperty(info, fileName, path_max());
                extLen = Recls_GetFileExtProperty(info, fileExt, path_max());
                printf("  %*s\n", off, file);
                printf("  %*s\n", off - (0 != extLen ? 1 + extLen : 0), fileName);
                printf("  %*s\n", off, fileExt);
                strcpy(fileCheck, fileName);
                if(0 < extLen)
                {
                    strcat(fileCheck, ".");
                    strcat(fileCheck, fileExt);
                }

                strcat(pathCheck, file);

                /* Now validate the components */
#if defined(RECLS_PLATFORM_IS_WIN32) && \
    !defined(RECLS_PLATFORM_API_WIN32)
                recls_assert(0 == strcmp(path + 2, pathCheck));
                if(0 != strcmp(path + 2, pathCheck))
#else /* ? RECLS_PLATFORM_API_WIN32 */
                recls_assert(0 == strcmp(path, pathCheck));
                if(0 != strcmp(path, pathCheck))
#endif /* RECLS_PLATFORM_API_WIN32 || !RECLS_PLATFORM_IS_WIN32 */
                {
                    fprintf(stderr, "Path is different from path components\n\tpath:  %s\n\tparts: %s\n\n", path, pathCheck);

                    abort();
                }
#if defined(RECLS_PLATFORM_API_WIN32)
                recls_assert(dirPath[0] == drive && 0 == strcmp(dir, dirPath + 2));
                if( dirPath[0] != drive ||
                    0 != strcmp(dir, dirPath + 2))
#elif defined(RECLS_PLATFORM_IS_WIN32)
                recls_assert(0 == strcmp(dir, dirPath + 2));
                if(0 != strcmp(dir, dirPath + 2))
#else
                recls_assert(0 == strcmp(dir, dirPath));
                if(0 != strcmp(dir, dirPath))
#endif /* RECLS_PLATFORM_API_WIN32 || !RECLS_PLATFORM_IS_WIN32 */
                {
                    fprintf(stderr, "DirectoryPath is different from Directory\n\tdirPath:  %s\n\tdir: %s\n\n", dirPath, dir);

                    abort();
                }
                if((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS)
                {
                    recls_assert(0 == strcmp(dir, dirCheck));
                    if(0 != strcmp(dir, dirCheck))
                    {
                        fprintf(stderr, "Directory is different from directory components\n\tpath:  %s\n\tparts: %s\n\n", dir, dirCheck);

                        abort();
                    }
                }
                recls_assert(0 == strcmp(file, fileCheck));
                if(0 != strcmp(file, fileCheck))
                {
                    fprintf(stderr, "File is different from file components\n\tpath:  %s\n\tparts: %s\n\n", file, fileCheck);

                    abort();
                }

                if(Recls_IsFileReadOnly(info))
                {
                    printf("    - Read-only\n");
                }
                if(Recls_IsFileDirectory(info))
                {
                    printf("    - Directory\n");
                }
                else
                {
                    printf("    - File\n");
                }
                if(Recls_IsFileLink(info))
                {
                    printf("    - Link\n");
                }
            }

            Recls_CloseDetails(info);
        }

        if(rc != RECLS_RC_NO_MORE_DATA)
        {
            recls_char_t    err[100];

            Recls_GetErrorString(rc, err, 100);

            fprintf(stderr, "Search terminated prematurely; recls error: %s\n", err);
        }

        Recls_SearchClose(hSrch);
    }

    return totalFound;
}

/* //////////////////////////////////////////////////////////////////////////
 * End of file
 */
