/* /////////////////////////////////////////////////////////////////////////////
 * File:        COM.cpp
 *
 * Purpose:     Implementation file for the COM project.
 *
 * Created:     19th December 2003
 * Updated:     10th January 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 2003-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/** \file COM.cpp Test program for the \ref group_recls_com "recls COM mapping" */

#include <windows.h>
#include <stdio.h>
#if defined(__DMC__)
# include <iostream.h>
#else /* ? __DMC__ */
# include <iostream>
# include <iomanip>

using std::cout;
using std::wcout;
using std::cerr;
using std::endl;
#endif /* __DMC__ */
#include <stdlib.h>

#include <stlsoft.h>

#include <atlstl.h>

#include <comstl.h>
#include <comstl_enumerator_policies.h>
#include <comstl_initialisers.h>
#include <comstl_interface_cast.h>
#include <comstl_value_policies.h>
#include <comstl_enum_sequence.h>
#include <comstl_coll_sequence.h>

// This version of recls has been released at the same time as STLSoft 1.7.1 beta 3, in which
// comstl_method_cast.h resides in the "inprogress" sub-directory.
#include <inprogress/comstl_method_cast.h>
// 
// Once STLSoft 1.7.1 is released, this file will reside in the main directory, so you should
// comment out the include above, and uncomment the one below
//#include <comstl_method_cast.h>

#include <winstl.h>
#include <winstl_char_conversions.h>

#include <recls_com.h>
#include <recls_com_i.c>

#ifdef _MSC_VER
# include <CrtDbg.h>
#endif /* _MSC_VER */

//#define USE_ENUMVARIANT
#define USE_ENUMENTRY

/* /////////////////////////////////////////////////////////////////////////////
 * Helper functions
 */

template<typename S>
S &operator <<(S &s, CComBSTR const &bstr)
{
    wchar_t const   *str = bstr;

    return s << str;
}

namespace comstl
{
    inline CComBSTR &transfer_resource(CComBSTR &r, BSTR v)
    {
        comstl_message_assert("Resource destination not empty!", NULL == r.m_str);
        r.m_str = v;

        return r;
    }
}

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason);
int processRoot(IFileSearch *search, CComBSTR const &rootDir, CComBSTR const &pattern, long flags, int bSuccinct, int bFTP);
int processRoot(IFtpSearch *search, CComBSTR const &host, CComBSTR const &username, CComBSTR const &password, CComBSTR const &rootDir, CComBSTR const &pattern, long flags, int bSuccinct, int bFTP);
int processCollection(ISearchCollection *psc, long flags, int bSuccinct, int bFTP);

/* /////////////////////////////////////////////////////////////////////////////
 * Main
 */

/** Program main */
int main(int argc, char *argv[])
{
#if 0
    ::Sleep(100000);
#endif /* 0 */

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemState    memState;
#endif /* _MSC_VER && _MSC_VER */
    int             iRet            =   0;
    int             i;
    int             totalFound      =   0;
    int             bAllHardDrives  =   false;
    char const      *host	        =   NULL;
    char const      *username       =   NULL;
    char const      *password       =   NULL;
    char const      *pattern        =   NULL;
    char const      *rootDir        =   NULL;
    long            flags           =   RECLS_F_RECURSIVE;
    int             bSuccinct       =   false;

    for(i = 1; i < argc; ++i)
    {
        const char  *arg    =   argv[i];

        if(arg[0] == '-')
        {
            if(arg[1] == '-')
            {
                /* -- arguments */
            }
            else
            {
                /* - arguments */
                switch(arg[1])
                {
                    case    '?':
                        usage(1, NULL);
                        break;
                    case    'R':    /* Do not recurse */
                        flags &= ~(RECLS_F_RECURSIVE);
                        break;
                    case    'p':    /* Show directory parts */
                        flags |= RECLS_F_DIRECTORY_PARTS;
                        break;
                    case    'f':    /* Find files */
                        flags |= RECLS_F_FILES;
                        break;
                    case    'd':    /* Find directories */
                        flags |= RECLS_F_DIRECTORIES;
                        break;
                    case    'h':    /* Searches from hard drives */
                        bAllHardDrives = true;
                        break;
                    case    's':    /* Show only the full path; WHEREIS functionality */
                        bSuccinct = true;
                        break;
                    case    'H':    /* FTP host */
                        host = arg + 2;
                        break;
                    case    'U':    /* FTP username */
                        username = arg + 2;
                        break;
                    case    'P':    /* FTP password */
                        password = arg + 2;
                        break;
                    default:
                        usage(1, "Invalid argument(s) specified");
                        break;
                }
            }
        }
        else
        {
            /* other arguments */
            if(NULL == pattern)
            {
                pattern = arg;
            }
            else if(NULL == rootDir)
            {
                rootDir = arg;
            }
            else
            {
                usage(1, "Invalid argument(s) specified");
            }
        }
    }

	if( NULL != password &&
		(	NULL == host ||
			NULL == username))
	{
		usage(1, "Must specify host and username if specifying password");
	}

	if( NULL != username &&
		NULL == host)
	{
		usage(1, "Must specify host if specifying username");
	}

	if( NULL != host &&
		bAllHardDrives)
	{
		usage(1, "-h flag meaningless to FTP searches");
	}

    if( NULL != host &&
		NULL == rootDir)
    {
		usage(1, "Must specify a root directory for FTP searches");
    }

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemCheckpoint(&memState);
#endif /* _MSC_VER && _MSC_VER */

    // Search for files if neither files or directories specified
    //
    // Even though this is not necessary, because the recls API provides the
    // same interpretation, it's best to be explicit.
    if(0 == (flags & (RECLS_F_FILES | RECLS_F_DIRECTORIES)))
    {
        flags |= RECLS_F_FILES;
    }

#if 0
    if(NULL == pattern)
    {
        pattern = Recls_GetWildcardsAll();
    }
#endif /* 0 */

    if(NULL == rootDir)
    {
        rootDir = ".";
    }

	if(NULL != host)
	{
        IFtpSearch             *search;
        comstl::com_initialiser init;
        HRESULT                 hr      =   ::CoCreateInstance(CLSID_FileSearch, NULL, CLSCTX_ALL, IID_IFtpSearch, (void**)&search);

        if(FAILED(hr))
        {
            cerr << "Failed to load the recls COM server! Please check your installation" << endl;
        }
        else
        {
//            CComBSTR    bstrPattern = comstl::method_cast<CComBSTR>(search, &IFileSearch::get_WildcardsAll);

			// The ATL conversion is not to be trusted for early versions, so we do it here.
			winstl::a2w	host_(host);
			winstl::a2w	username_(username);
			winstl::a2w	password_(password);
			winstl::a2w	rootDir_(rootDir);
			winstl::a2w	pattern_(pattern);

			CComBSTR	bstrHost(stlsoft::c_str_ptr(host));
			CComBSTR	bstrUsername(stlsoft::c_str_ptr(username));
			CComBSTR	bstrPassword(stlsoft::c_str_ptr(password));
			CComBSTR	bstrRootDir(stlsoft::c_str_ptr(rootDir));
			CComBSTR	bstrPattern(stlsoft::c_str_ptr(pattern));

			if(0 == stlsoft::c_str_len(bstrPattern))
			{
				bstrPattern = L"*";
			}

            totalFound = processRoot(search, bstrHost, bstrUsername, bstrPassword, bstrRootDir, bstrPattern, flags, bSuccinct, true);

            search->Release();
		}
	}
	else
    {
        IFileSearch             *search;
        comstl::com_initialiser init;
        HRESULT                 hr      =   ::CoCreateInstance(CLSID_FileSearch, NULL, CLSCTX_ALL, IID_IFileSearch, (void**)&search);

        if(FAILED(hr))
        {
            cerr << "Failed to load the recls COM server! Please check your installation" << endl;
        }
        else
        {
			winstl::a2w	pattern_(pattern);
			CComBSTR	bstrPattern(stlsoft::c_str_ptr(pattern));

			if(0 == stlsoft::c_str_len(bstrPattern))
			{
				bstrPattern = comstl::method_cast<CComBSTR>(search, &IFileSearch::get_WildcardsAll);;
			}

            if(bAllHardDrives)
            {
                LPUNKNOWN   punkRoots;

                hr = search->get_Roots(&punkRoots);

                if(SUCCEEDED(hr))
                {
                    typedef comstl::enumerator_sequence<IEnumString, LPOLESTR, comstl::LPOLESTR_policy> enum_sequence_t;

                    enum_sequence_t             roots(comstl::interface_cast_addref<LPENUMSTRING>(punkRoots), false);
                    enum_sequence_t::iterator   begin   =   roots.begin();
                    enum_sequence_t::iterator	end     =   roots.end();

                    for(; begin != end; ++begin)
                    {
                        totalFound += processRoot(search, *begin, bstrPattern, flags, bSuccinct, false);
                    }

                    punkRoots->Release();
                }
            }
            else
            {
				winstl::a2w	rootDir_(rootDir);
				CComBSTR	bstrRootDir(stlsoft::c_str_ptr(rootDir));

                totalFound = processRoot(search, bstrRootDir, bstrPattern, flags, bSuccinct, false);
            }

            search->Release();
        }
    }

    fprintf(stdout, "Total found: %d\n", totalFound);

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemDumpAllObjectsSince(&memState);
#endif /* _MSC_VER && _MSC_VER */

    return iRet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Function implementations
 */

/** Displays program usage */
void usage(int bExit, char const *reason)
{
    cerr << "recls COM (C++) Test Program: COM" << endl << endl;

	if(NULL != reason)
	{
		cerr << "  Error: " << reason << endl << endl;
	}

    cerr << "Usage: COM [-d] [-f] [-h] [-p] [-R] [-s] [-H<host>] [-U<username>] [-P<password>] [<pattern>] [<root-dir>]" << endl;
    cerr << "\t-R               -   does not recurse; recursive search is the default" << endl;
    cerr << "\t-p               -   evaluate and display directory parts" << endl;
    cerr << "\t-d               -   search for directories" << endl;
    cerr << "\t-f               -   search for files (this is the default, if -d is not specified" << endl;
    cerr << "\t-h               -   searches from hard drive roots. (Ignore <root-dir>.)" << endl;
    cerr << "\t-s               -   succinct; shows only the path" << endl;
	cerr << "\t-H<host>         -   performs an FTP search on the given host" << endl;
	cerr << "\t-U<username>     -   username to use to log on to <host>" << endl;
	cerr << "\t-P<password>     -   password to use to log on to <host>" << endl;
    cerr << "\t<pattern>        -   search pattern, e.g. \"*.cpp\"; default is to search for all files" << endl;
    cerr << "\t<root-dir>       -   root directory of search; default is current working directory" << endl;

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

int processRoot(IFtpSearch *search, CComBSTR const &host, CComBSTR const &username, CComBSTR const &password, CComBSTR const &rootDir, CComBSTR const &pattern, long flags, int bSuccinct, int bFTP)
{
    int         totalFound  =   0;
    IUnknown    *punkResults;
    HRESULT     hr          =   search->Search(host, username, password, rootDir, pattern, flags, &punkResults);

    if(SUCCEEDED(hr))
    {
        ISearchCollection   *psc = comstl::interface_cast_addref<ISearchCollection*>(punkResults);

        if(NULL != psc)
        {
			totalFound = processCollection(psc, flags, bSuccinct, bFTP);

            psc->Release();
        }

        punkResults->Release();
    }

    return totalFound;
}

int processRoot(IFileSearch *search, CComBSTR const &rootDir, CComBSTR const &pattern, long flags, int bSuccinct, int bFTP)
{
    int         totalFound  =   0;
    IUnknown    *punkResults;
    HRESULT     hr          =   search->Search(rootDir, pattern, flags, &punkResults);

    if(SUCCEEDED(hr))
    {
        ISearchCollection   *psc = comstl::interface_cast_addref<ISearchCollection*>(punkResults);

        if(NULL != psc)
        {
			totalFound = processCollection(psc, flags, bSuccinct, bFTP);

            psc->Release();
        }

        punkResults->Release();
    }

    return totalFound;
}

int processCollection(ISearchCollection *psc, long flags, int bSuccinct, int bFTP)
{
    int             totalFound      =   0;

#if defined(USE_ENUMVARIANT)
    typedef comstl::collection_sequence<    ISearchCollection
                                        ,   IEnumVARIANT
                                        ,   VARIANT
                                        ,   comstl::VARIANT_policy
                                        >           coll_seq_t;
#elif defined(USE_ENUMENTRY)
    typedef comstl::collection_sequence<    ISearchCollection
                                        ,   IEnumFileEntry
                                        ,   IFileEntry*
                                        ,   comstl::interface_policy<IFileEntry>
                                        >           coll_seq_t;
#else
# error Must specify USE_ENUMVARIANT or USE_ENUMENTRY
#endif /* 0 */

    coll_seq_t              collection(psc, true, bFTP ? 1 : 0);
    coll_seq_t::iterator    b   =   collection.begin();
    coll_seq_t::iterator    e   =   collection.end();

    for(; b != e; ++b)
    {
        ++totalFound;

#if defined(USE_ENUMVARIANT)
        IFileEntry  *pfe = comstl::interface_cast_addref<IFileEntry*>((*b).punkVal);
#elif defined(USE_ENUMENTRY)
        IFileEntry  *pfe = *b;
#else
# error Must specify USE_ENUMVARIANT or USE_ENUMENTRY
#endif /* 0 */

        CComBSTR    path                    =   comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_Path);
        OLECHAR     drive                   =   comstl::method_cast<OLECHAR>(pfe, &IFileEntry::get_Drive);
        CComBSTR    directory               =   comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_Directory);
        CComBSTR    directoryPath           =   comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_DirectoryPath);
        LPUNKNOWN   directoryParts          =   comstl::method_cast<LPUNKNOWN>(pfe, &IFileEntry::get_DirectoryParts);
        CComBSTR    file                    =   comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_File);
        CComBSTR    shortFile               =   comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_ShortFile);
        CComBSTR    fileName                =   comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_FileName);
        CComBSTR    fileExt                 =   comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_FileExt);
        DATE        creationTime            =   comstl::method_cast<DATE>(pfe, &IFileEntry::get_CreationTime);
        DATE        modificationTime        =   comstl::method_cast<DATE>(pfe, &IFileEntry::get_ModificationTime);
        DATE        lastAccessTime          =   comstl::method_cast<DATE>(pfe, &IFileEntry::get_LastAccessTime);
        DATE        lastStatusChangeTime    =   comstl::method_cast<DATE>(pfe, &IFileEntry::get_LastStatusChangeTime);
        long        size                    =   comstl::method_cast<long>(pfe, &IFileEntry::get_Size);
        BOOL        isReadOnly              =   comstl::method_cast<BOOL>(pfe, &IFileEntry::get_IsReadOnly);
        BOOL        isDirectory             =   comstl::method_cast<BOOL>(pfe, &IFileEntry::get_IsDirectory);

        wcout << L"  " << path << endl;

        if(!bSuccinct)
        {
            wcout << L"  " << directoryPath << endl;
            wcout << L"  " << (wchar_t const *)winstl::a2w((char const*)&drive, 1) << endl;
            wcout << L"    " << directory << endl;

            int off = 0;

            if( (flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS &&
                NULL != directoryParts)
            {
                typedef comstl::collection_sequence<    IDirectoryPartsCollection
                                                    ,   IEnumVARIANT
                                                    ,   VARIANT
                                                    ,   comstl::VARIANT_policy
                                                    ,   VARIANT const &
                                                    ,   comstl::forward_cloning_policy<IEnumVARIANT>
                                                    ,   2
                                                    >           parts_seq_t;

                parts_seq_t             dirParts(comstl::interface_cast_addref<IDirectoryPartsCollection*>(directoryParts), false);
                parts_seq_t::iterator   b   =   dirParts.begin();
                parts_seq_t::iterator   e   =   dirParts.end();

                for(; b != e; ++b)
                {
                    CComVariant vPart(*b);  vPart.ChangeType(VT_BSTR);
                    CComBSTR    dirPart(vPart.bstrVal);

                    off += dirPart.Length();
                    wcout << L"    " << std::setw(off) << dirPart << std::setw(1) << endl;
                }

                directoryParts->Release();
            }
            else
            {
                off += directory.Length();
            }

            off += file.Length();
            wcout << L"    " << std::setw(off) << file << std::setw(1) << endl;
            if(fileExt.Length() > 0)
            {
                wcout << L"    " << std::setw(off - (1 + fileExt.Length())) << fileName << std::setw(1) << endl;
                wcout << L"    " << std::setw(off) << fileExt << std::setw(1) << endl;
            }
            else
            {
                wcout << L"    " << std::setw(off) << fileName << std::setw(1) << endl;
            }

            if(isReadOnly)
            {
                wcout << L"    - Read-only" << endl;
            }
            if(isDirectory)
            {
                wcout << L"    - Directory" << endl;
            }
            else
            {
                wcout << L"    - File" << endl;
            }
        }

#if defined(USE_ENUMVARIANT)
        pfe->Release();
#endif /* USE_ENUMVARIANT */
    }

	return totalFound;
}


/* //////////////////////////////////////////////////////////////////////////
 * End of file
 */
