/* /////////////////////////////////////////////////////////////////////////////
 * File:        Cpp.cpp
 *
 * Purpose:     Implementation file for the Cpp project.
 *
 * Created:     16th August 2003
 * Updated:     5th March 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 2003-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 */

/** \file Cpp.cpp Test program for the \ref group_recls_cpp "recls C++ mapping" */

/* Define this definition to retrict use to platform-independent aspects of the API */
//#define RECLS_PURE_API

#include <stdio.h>

#include <recls.h>

#if RECLS_VER < RECLS_VER_1_1_1
# error This file now requires version 1.1.1 or later of the recls C API
#endif /* RECLS_VER < 1.1.1 */

#if defined(RECLS_STRICT) && \
    !defined(RECLS_COMPILER_IS_DMC)
# include <stlsoft_nulldef.h>
#endif /* RECLS_STRICT && !RECLS_COMPILER_IS_DMC */

#ifdef RECLS_COMPILER_IS_MSVC
# include <crtdbg.h>
# pragma warning(disable : 4530)
#endif /* _MSC_VER */

#if RECLS_VER >= RECLS_VER_1_5_1 && \
    defined(RECLS_PLATFORM_IS_WIN32)
# define RECLS_TEST_CPP_USING_FTP
#endif /* RECLS_VER >= RECLS_VER_1_5_1 && RECLS_PLATFORM_IS_WIN32 */

#include <recls_assert.h>
#include <recls/cpp/filesearch.hpp>
#if defined(RECLS_TEST_CPP_USING_FTP)
# include <recls/cpp/ftpsearch.hpp>
#endif /* RECLS_TEST_CPP_USING_FTP */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

using recls::recls_uint32_t;
using recls::recls_char_t;
using recls::RECLS_F_FILES;
using recls::RECLS_F_DIRECTORIES;
using recls::RECLS_F_RECURSIVE;
using recls::RECLS_F_DIRECTORY_PARTS;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason);
static int processSearch(reclspp::Search &search, int bSuccinct, recls_uint32_t flags);

/* /////////////////////////////////////////////////////////////////////////////
 * Main
 */

/** Program main */
int main(int argc, char *argv[])
{
#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemState	memState;
#endif /* _MSC_VER && _MSC_VER */
    int             iRet			=   0;
    int             i;
    int             totalFound;
    int             bAllHardDrives  =   0;
    char const      *host	        =   NULL;
    char const      *username       =   NULL;
    char const      *password       =   NULL;
    char const      *pattern		=   NULL;
    char const      *rootDir		=   NULL;
    recls_uint32_t  flags			=   RECLS_F_RECURSIVE;
    int             bSuccinct		=   false;

    for(i = 1; i < argc; ++i)
    {
        const char  *arg    =   argv[i];

        if(arg[0] == '-')
        {
            if(arg[1] == '-')
            {
                /* -- arguments */
            }
            else
            {
                /* - arguments */
                switch(arg[1])
                {
                    case    '?':
                        usage(1, NULL);
                        break;
                    case    'R':    /* Do not recurse */
                        flags &= ~(RECLS_F_RECURSIVE);
                        break;
                    case    'p':    /* Show directory parts */
                        flags |= RECLS_F_DIRECTORY_PARTS;
                        break;
                    case    'f':    /* Find files */
                        flags |= RECLS_F_FILES;
                        break;
                    case    'd':    /* Find directories */
                        flags |= RECLS_F_DIRECTORIES;
                        break;
                    case    'h':    /* Searches from hard drives */
                        bAllHardDrives = true;
                        break;
                    case    's':    /* Show only the full path; WHEREIS functionality */
                        bSuccinct = true;
                        break;
                    case    'H':    /* FTP host */
                        host = arg + 2;
                        break;
                    case    'U':    /* FTP username */
                        username = arg + 2;
                        break;
                    case    'P':    /* FTP password */
                        password = arg + 2;
                        break;
                    default:
                        usage(1, "Invalid argument(s) specified");
                        break;
                }
            }
        }
        else
        {
            /* other arguments */
            if(NULL == pattern)
            {
                pattern = arg;
            }
            else if(NULL == rootDir)
            {
                rootDir = arg;
            }
            else
            {
                usage(1, "Invalid argument(s) specified");
            }
        }
    }

	if( NULL != password &&
		(	NULL == host ||
			NULL == username))
	{
		usage(1, "Must specify host and username if specifying password");
	}

	if( NULL != username &&
		NULL == host)
	{
		usage(1, "Must specify host if specifying username");
	}

	if( NULL != host &&
		bAllHardDrives)
	{
		usage(1, "-h flag meaningless to FTP searches");
	}

    if( NULL != host &&
		NULL == rootDir)
    {
		usage(1, "Must specify a root directory for FTP searches");
    }

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemCheckpoint(&memState);
#endif /* _MSC_VER && _MSC_VER */

    // Search for files if neither files or directories specified
    //
    // Even though this is not necessary, because the recls API provides the
    // same interpretation, it's best to be explicit.
    if(0 == (flags & (RECLS_F_FILES | RECLS_F_DIRECTORIES)))
    {
        flags |= RECLS_F_FILES;
    }

	/* Now conduct the search */
#if defined(RECLS_TEST_CPP_USING_FTP)
	if(NULL != host)
    {
		if(NULL == pattern)
		{
			pattern = recls::Recls_GetWildcardsAll();
		}

		if(NULL == rootDir)
		{
			rootDir = "/";
		}

		reclspp::FtpSearch	search(host, username, password, rootDir, pattern, flags);

		totalFound = processSearch(search, bSuccinct, flags);
	}
	else
#endif /* RECLS_TEST_CPP_USING_FTP */
	{
		if(NULL == pattern)
		{
			pattern = recls::Recls_GetWildcardsAll();
		}

		if(NULL == rootDir)
		{
			rootDir = ".";
		}

		reclspp::FileSearch	search(rootDir, pattern, flags);

		totalFound = processSearch(search, bSuccinct, flags);
	}

    fprintf(stdout, "Total found: %d\n", totalFound);

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemDumpAllObjectsSince(&memState);
#endif /* _MSC_VER && _MSC_VER */

    return iRet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Function implementations
 */

/** Displays program usage */
static void usage(int bExit, char const *reason)
{
    fprintf(stderr, "recls C++ Test Program\n\n");

	if(NULL != reason)
	{
		fprintf(stderr, "  Error: %s\n\n", reason);
	}

    fprintf(stderr, "Usage: Cpp [-d] [-f] [-h] [-p] [-R] [-s] [-H<host>] [-U<username>] [-P<password>] [<pattern>] [<root-dir>]\n");
    fprintf(stderr, "\t-R               -   does not recurse; recursive search is the default\n");
    fprintf(stderr, "\t-p               -   evaluate and display directory parts\n");
    fprintf(stderr, "\t-d               -   search for directories\n");
    fprintf(stderr, "\t-f               -   search for files (this is the default, if -d is not specified)\n");
    fprintf(stderr, "\t-h               -   searches from hard drive roots. (Ignore <root-dir>.)\n");
    fprintf(stderr, "\t-s               -   succinct; shows only the path\n");
	fprintf(stderr, "\t-H<host>         -   performs an FTP search on the given host\n");
	fprintf(stderr, "\t-U<username>     -   username to use to log on to <host>\n");
	fprintf(stderr, "\t-P<password>     -   password to use to log on to <host>\n");
    fprintf(stderr, "\t<pattern>        -   search pattern, e.g. \"*.cpp\"; default is to search for all files\n");
    fprintf(stderr, "\t<root-dir>       -   root directory of search; default is current working directory\n");

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

/* /////////////////////////////////////////////////////////////////////////////
 * Function implementations
 */

static int processSearch(reclspp::Search &search, int bSuccinct, recls_uint32_t flags)
{
	int	totalFound	=	0;

    for(; search.HasMoreElements(); search.GetNext())
    {
        using reclspp::string_t;

        ++totalFound;

        reclspp::FileEntry      fileEntry   =   search.GetCurrentEntry();
#ifdef RECLS_PLATFORM_API_WIN32
        recls_char_t            drive       =   fileEntry.GetDrive();
#endif /* RECLS_PLATFORM_API_WIN32 */
        string_t                path        =   fileEntry.GetPath();
        string_t                dir         =   fileEntry.GetDirectory();
        string_t                dirPath     =   fileEntry.GetDirectoryPath();
        reclspp::DirectoryParts dirParts    =   fileEntry.GetDirectoryParts();
        string_t                file        =   fileEntry.GetFile();
        string_t                fileName    =   fileEntry.GetFileName();
        string_t                fileExt     =   fileEntry.GetFileExt();

        string_t                pathCheck;
        string_t                dirPartsCheck;
        string_t                fileCheck;

#ifdef x_DEBUG
        reclspp::FileEntry  entry2;

        if( fileEntry
            && !entry2)
        {
            fprintf(stderr, "Good entry\n");
        }
        if(!fileEntry)
        {
            fprintf(stderr, "Bad entry\n");
        }
#endif /* _DEBUG */

#ifdef _DEBUG
        {
            recls_uint32_t  cBlocks = search.GetNumOutstandingDetails();

            printf("\n%lu outstanding block(s)\n", cBlocks);
        }
#endif /* _DEBUG */

        printf("  %s\n", path.c_str());
        if(!bSuccinct)
        {
            int     off =   0;

            printf("  %s\n", dirPath.c_str());
#ifdef RECLS_PLATFORM_API_WIN32
			if(0 != drive)
			{
				if(isupper(path[0]))
				{
					drive = (recls_char_t)toupper(drive);
				}
				else
				{
					drive = (recls_char_t)tolower(drive);
				}
				printf("  %c\n", drive);
				pathCheck += drive;
				pathCheck += ':';

				off += 2;
			}
#endif /* RECLS_PLATFORM_API_WIN32 */
            printf("    %s\n", dir.c_str());

            if((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS)
            {
                int     i;
                int     cDirParts;

                for(i = 0, cDirParts = dirParts.size(); i < cDirParts; ++i)
                {
                    string_t    dirPart =   dirParts[i];

                    off += dirPart.length();
                    printf("    %*s\n", off, dirPart.c_str());

                    pathCheck += dirPart;
                    dirPartsCheck += dirPart;
                }
            }
            else
            {
                off += dir.length();
                pathCheck += dir;
            }

            off += file.length();
            printf("    %*s\n", off, file.c_str());
            if(fileExt.length() > 0)
            {
                printf("    %*s\n", off - (1 + fileExt.length()), fileName.c_str());
                printf("    %*s\n", off, fileExt.c_str());
            }
            else
            {
                printf("    %*s\n", off, fileName.c_str());
            }

            fileCheck = fileName;
            if(0 < fileExt.length())
            {
                fileCheck += '.';
                fileCheck += fileExt;
            }

            pathCheck += file;

            // Now validate the components
            recls_assert(path == pathCheck);
            if(path != pathCheck)
            {
                fprintf(stderr, "Path is different from path components\n\tpath:  %s\n\tparts: %s\n\n", path.c_str(), pathCheck.c_str());

                abort();
            }
#ifdef RECLS_PLATFORM_API_WIN32
            recls_assert(0 == drive || (dirPath[0] == drive && dir == dirPath.c_str() + 2));
            recls_assert(0 != drive || dir == dirPath);
            if( (	0 != drive &&
					(	dirPath[0] != drive ||
						dir != dirPath.c_str() + 2)) ||
				(	0 == drive &&
					dir != dirPath))
#else
            recls_assert(dirPath == dir);
            if(dirPath != dir)
#endif /* RECLS_PLATFORM_API_WIN32 */
            {
                fprintf(stderr, "DirectoryPath is different from Directory\n\tdirPath:  %s\n\tdir: %s\n\n", dirPath.c_str(), dir.c_str());

                abort();
            }
            if((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS)
            {
                recls_assert(dir == dirPartsCheck);
                if(dir != dirPartsCheck)
                {
                    fprintf(stderr, "Directory is different from directory components\n\tpath:  %s\n\tparts: %s\n\n", dir.c_str(), dirPartsCheck.c_str());

                    abort();
                }
            }
            recls_assert(file == fileCheck);
            if(file != fileCheck)
            {
                fprintf(stderr, "File is different from file components\n\tpath:  %s\n\tparts: %s\n\n", file.c_str(), fileCheck.c_str());

                abort();
            }

            if(fileEntry.IsReadOnly())
            {
                printf("    - Read-only\n");
            }
            if(fileEntry.IsDirectory())
            {
                printf("    - Directory\n");
            }
            else
            {
                printf("    - File\n");
            }
            if(fileEntry.IsLink())
            {
                printf("    - Link\n");
            }
        }
	}

	return totalFound;
}

/* //////////////////////////////////////////////////////////////////////////
 * End of file
 */
