
// Updated: 6th March 2005

/** \mainpage
 *
 * \section mainpage_introduction Introduction
 *
 * This is the documentation for the <b>recls</b> library. (Click \ref releases "here" for release history.)
 *
 * <b>recls</b> (http://recls.org/) is a C library (implemented in C & C++) that provides recursive 
 * file-system \htmlonly<sup><b><a href = "#1">1</a></b></sup>\endhtmlonly searching. It is described in my 
 * "<b>Positive Integration</b>" column, in <a href = "http://www.cuj.com">C/C++ User's Journal (CUJ)</a>.
 *
 * The purpose of the library is twofold. As well as providing an educational experience for the readers
 * (and the author!) of the CUJ column, it is also intended to be a production-quality library, and it is
 * my hope that it will find use throughout the development community. The 
 * <a href = "http://www.synesis.com.au/licenses/ssssl.html">license</a> is very open, and allows use in any
 * commercial or non-commercial organisation, without requiring provision of source, or runtime acknowledgements.
 *
 * \section mainpage_mappings Language Mappings
 *
 * In addition to the \ref group_recls "recls C API", the <b>recls</b> library includes the following language mappings:
 *
 * - \ref group_recls_cpp <b>C++</b>: A set of simple classes that provide a user-friendly interface over the \ref group_recls "recls C" API
 * - \ref group_recls_cs <b>C#</b>: A <a href = "http://www.microsoft.com/net">.NET</a> mapping written in C#.
 * - \ref group_recls_com <b>COM</b>: A mapping to a <a href = "http://www.microsoft.com/com">COM</a> Collection, written in ATL.
 * - \ref group_recls_d <b>D</b>: A mapping to <a href = "http://www.digitalmars.com/d/">D</a>.
 * - \ref group_recls_java <b>Java</b>: A mapping to <a href = "http://java.sun.com/">Java</a>, demonstrating JNI (Java Native Interface).
 * - \ref group_recls_ruby <b>Ruby</b>: A mapping to <a href = "http://ruby-lang.org/">Ruby</a>, demonstrating Ruby extensions.
 * - \ref group_recls_stl <b>STL</b>: A mapping of the \ref group_recls "recls C" API to C++, providing <a href = "http://sgi.com/tech/stl/">STL</a>-like sequences representing matching file ranges, and directory sub-components.
 *
 *
 * Several other mappings will be provided, as the library evolves through the column:
 *
 * - <b>Ch</b>
 * - <b>Heron</b>
 * - <b>Managed C++</b>: A <a href = "http://www.microsoft.com/net">.NET</a> mapping written in Managed C++
 * - <b>Perl</b>
 * - <b>Python</b>
 * - <b>Tcl</b>
 *
 * \section mainpage_installation Installation
 *
 * At the moment, the libraries are provided in source (+ project) form only; there are no binaries. This means
 * that you are required to build the libraries yourself. Since the project is primarily educational, this shouldn't
 * present an issue (I hope).
 *
 * Building the libraries requires that you have <a href = "http://stlsoft.org/downloads.html">version 1.6.5</a> or 
 * later of the <a href = "http://stlsoft.org/">STLSoft</a> libraries, and that you've defined the enviroment variable
 * <code><b>STLSOFT_INCLUDE</b></code> to refer to the STLSoft directory.
 *
 * Once other language mappings, that rely on dynamic libraries (such as the .NET ones), then it is possible that
 * binary versions will be made available.
 *
 * \section mainpage_compilers Compilers
 *
 * At the current time, the supported compilers are:
 *
 * - <a href = "http://www.borland.com">Borland C/C++</a> (versions 5.5.1, 5.6 & 5.6.4)
 * - <a href = "http://www.comeaucomputing.com">Comeau C++</a>, (version 4.3.0.1 & 4.3.3)
 * - <a href = "http://www.digitalmars.com">Digital Mars C/C++</a> (versions 8.35 and above)
 * - <a href = "http://gcc.gnu.org">GNU C/C++</a> (versions 3.2 and above)
 * - <a href = "http://www.intel.com">Intel C/C++</a> (versions 6.0, 7.0, 7.1 & 8.0)
 * - <a href = "http://www.metrowerks.com">Metrowerks Code Warrior</a> (versions 7.0, 8.0 & 9.0)
 * - <a href = "http://www.microsoft.com">Microsoft Visual C/C++</a> (versions 5.0, 6.0, 7.0, 7.1 & 8.0)
 *
 * Note that for version 1.0.1, makefiles are provided for Borland C++, CodeWarrior, Digital Mars, GCC and
 * Visual C++ only.
 *
 * \section mainpage_acknowledgements Acknowledgements
 *
 * Many thanks are due to <b>Chuck Allison</b>, <b>Joe Casad</b> & <b>Jon Erickson</b>, and 
 * <a href = "http://www.cuj.com">C/C++ User's Journal</a> & <a href = "http://cmp.com">CMP</a>
 * publishing, without whose faith in my ability to steward the "<b>Positive Integration</b>"
 * column, the <b>recls</b> project wouldn't exist.
 *
 * Thanks are also due to the following, who've lent support in one way or another:
 *
 * - <b>David Hackson</b> of <a href = "http://www.intel.com">Intel</a>
 * - <b>Greg Peet</b> (<a href = "http://gregpeet.com/">gregpeet.com</a>)
 * - <b>Herb Sutter</b> (<a href = "http://www.gotw.ca/">www.gotw.ca</a>)
 * - <b>Jeff Tieszen</b> of <a href = "http://www.metrowerks.com">Metrowerks</a>
 * - <b>Walter Bright</b> of <a href = "http://www.digitalmars.com/">Digital Mars</a>
 * - <b>Yukihiro Matsumoto</b> of <a href = "http://www.ruby-lang.org/">Ruby</a>
 * - <a href = "http://www.microsoft.com/"><b>Microsoft</b></a>
 * - <a href = "http://www.borland.com/"><b>Borland</b></a>
 *
 * <br>
 * Best regards, and good coding
 *
 * <b>Matthew Wilson</b>
 * 
 * \htmlonly
  <table border = "0">
   <tr>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1">Author, <a target = "top" href = "http://www.imperfectcplusplus.com/">"Imperfect C++"</a>, Addison-Wesley, 2004</font></td>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1">(<a target = "top" href = "http://www.imperfectcplusplus.com/">http://www.imperfectcplusplus.com</a>)</font></td>
   </tr>
   <tr>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1">Contributing editor, <a target = "top" href = "http://www.cuj.com/">C/C++ Users Journal</a></font></td>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1">(<a target = "top" href = "http://www.synesis.com.au/articles.html#columns">http://www.synesis.com.au/articles.html#columns</a>)</font></td>
   </tr>
   <tr>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1"><b>recls</b> implementor</font></td>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1"><b>admin</b> <i>hat</i> <b>stlsoft</b> <i>dot</i> <b>org</b></font></td>
   </tr>
   <tr>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1">Director, Synesis Software</font></td>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1">(<a target = "top" href = "http://www.synesis.com.au">http://www.synesis.com.au</a>)</font></td>
   </tr>
   <tr>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1">STLSoft moderator and C++ monomaniac</font></td>
    <td>&nbsp;&nbsp;</td>
    <td><font size = "-1">(<a target = "top" href = "http://www.stlsoft.org/">http://www.stlsoft.org/</a>)</font></td>
   </tr>
  </table>
  \endhtmlonly
 *
 * <hr>
 */

