
// Updated: 18th July 2004

/// The Ruby recls module
namespace Recls
{
	/// The recls version from which this Ruby mapping was made
	///
	/// \ingroup group_recls_ruby
	STRING RECLS_VERSION;
	/// The version of this Ruby mapping
	///
	/// \ingroup group_recls_ruby
	STRING VERSION;
	/// Include files in search. Included by default if none specified
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_FILES;
	/// Include directories in search
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_DIRECTORIES;
	/// Include links in search. Ignored in Win32
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_LINKS;
	/// Include devices in search. Not currently supported
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_DEVICES;
	/// Searches given directory and all sub-directories
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_RECURSIVE;
	/// Does not expand links
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_NO_FOLLOW_LINKS;
	/// Fills out the directory parts. Supported from version 1.1.1 onwards
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_DIRECTORY_PARTS;
	/// Does not fill out anything other than the path
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_DETAILS_LATER;
	/// Passive mode in FTP. Supported from version 1.5.1 onwards
	///
	/// \ingroup group_recls_ruby
	INT RECLS_F_PASSIVE_FTP;
		
	/// A file object
	class Entry
	{
	public:
		/// The path attribute
		STRING path;
		/// The drive attribute
		STRING drive;
		/// The directory attribute
		STRING directory;
		/// The directoryPath attribute
		STRING directoryPath;
		/// The directoryParts attribute
		ARRAY directoryParts;
		/// The file attribute
		STRING file;
		/// The shortFile attribute
		STRING shortFile;
		/// The fileName attribute
		STRING fileName;
		/// The fileExt attribute
		STRING fileExt;
		/// The creationTime attribute
		DATE creationTime;
		/// The modificationTime attribute
		DATE modificationTime;
		/// The lastAccessTime attribute
		DATE lastAccessTime;
		/// The lastStatusChangeTime attribute
		DATE lastStatusChangeTime;
		/// The size attribute
		INT size;
		/// The isReadOnly attribute
		INT isReadOnly;
		/// The isDirectory attribute
		INT isDirectory;
		/// The isLink attribute
		INT isLink;
		/// The to_s method
		STRING to_s;
	};
		
	/// This class provides file-system searching
	class FileSearch
	{
	public:
		/// Constructs an instance of the class	
		FileSearch(STRING rootDir, STRING pattern, INT flags);

		/// The each enumerator method
		void each(_block_);

		/// The searchRoot attribute
		STRING searchRoot;

		/// The pattern attribute
		STRING pattern;

		/// The flags attribute
		INT flags;
	};

	/// This class provides FTP searching on an internet host
	class FtpSearch
	{
	public:
		/// Constructs an instance of the class	
		FtpSearch(STRING host, STRING username, STRING password, STRING rootDir, STRING pattern, INT flags);

		/// The each enumerator method
		void each(_block_);

		/// The searchRoot attribute
		STRING searchRoot;

		/// The pattern attribute
		STRING pattern;

		/// The flags attribute
		INT flags;
	};

}


