
toolsDir	=	ENV["STLSOFT_TOOLS"]

if !toolsDir
	puts "STLSOFT_TOOLS environment variable is not defined"
	abort
end

$: << toolsDir

require "recls"
require "common"

reclsDir	=	ENV["RECLS_DEV"]

if !reclsDir
	puts "RECLS_DEV must be defined in order to run this script"
	abort
end

class Project
	def initialize(name, desc, directories, dirDepth)
		@name = name
		@desc = desc
		@dirDepth = dirDepth
		@directories = directories
	  	def name
		    @name
		end
  		def desc
		    @desc
		end
  		def directories
		    @directories
		end
  		def dirDepth
		    @dirDepth
		end
	end
end

lib_target = Project::new("lib_target", "recls library", [ "$(RECLS_API_BUILD_DIR)", "$(COMP_TAG)"], 3 )
map_DLL    = Project::new("map_DLL", "DLL mapping", [ "$(RECLS_MAPPINGS_DIR)", "DLL", "$(COMP_TAG)"], 3 )
map_COM    = Project::new("map_COM", "COM mapping", [ "$(RECLS_MAPPINGS_DIR)", "COM", "$(COMP_TAG)"], 3 )
map_CS     = Project::new("map_CS", "C-Sharp mapping", [ "$(RECLS_MAPPINGS_DIR)", "CS"], 2 )
map_D      = Project::new("map_D", "D mapping", [ "$(RECLS_MAPPINGS_DIR)", "D"], 2 )
map_Java   = Project::new("map_Java", "Java mapping", [ "$(RECLS_MAPPINGS_DIR)", "Java"], 2 )
map_Perl   = Project::new("map_Perl", "Perl mapping", [ "$(RECLS_MAPPINGS_DIR)", "Perl"], 2 )
map_Python = Project::new("map_Python", "Python mapping", [ "$(RECLS_MAPPINGS_DIR)", "Python"], 2 )
map_Ruby   = Project::new("map_Ruby", "Ruby mapping", [ "$(RECLS_MAPPINGS_DIR)", "Ruby"], 2 )

test_C     = Project::new("test_C", "C test program", [ "$(RECLS_API_TEST_DIR)", "C", "$(COMP_TAG)"], 3 )
test_Cpp   = Project::new("test_Cpp", "C++ test program", [ "$(RECLS_API_TEST_DIR)", "Cpp", "$(COMP_TAG)"], 3 )
test_COM   = Project::new("test_COM", "COM test program", [ "$(RECLS_API_TEST_DIR)", "COM", "$(COMP_TAG)"], 3 )
test_CS    = Project::new("test_CS", "C-Sharp test program", [ "$(RECLS_API_TEST_DIR)", "CS"], 2 )
test_D     = Project::new("test_D", "D test program", [ "$(RECLS_API_TEST_DIR)", "D"], 2 )
test_Java  = Project::new("test_Java", "Java test program", [ "$(RECLS_API_TEST_DIR)", "Java"], 2 )
test_SearchProcess = Project::new("test_SearchProcess", "SearchProcess test program", [ "$(RECLS_API_TEST_DIR)", "SearchProcess_test", "$(COMP_TAG)"], 3 )
test_STL   = Project::new("test_STL", "STL test program", [ "$(RECLS_API_TEST_DIR)", "STL", "$(COMP_TAG)"], 3 )


class Compiler
	def initialize(name, desc, make, suffix, projects)
		@name = name
		@desc = desc
		@make = make
		@suffix = suffix
		@projects = projects
	  	def name
		    @name
		end
  		def desc
		    @desc
		end
	  	def make
		    @make
		end
	  	def suffix
		    @suffix
		end
	  	def projects
		    @projects
		end
	end
end

bc56          = Compiler::new("bc56", "Borland 5.6", "make", "", [ lib_target, test_C, test_Cpp, test_SearchProcess, test_STL ])
como433_win32 = Compiler::new("como433", "Comeau 4.3.3", "make", "win32", [ lib_target, test_C, test_Cpp, test_SearchProcess ])
cw8_win32     = Compiler::new("cw8", "Metrowerks CodeWarrior 8", "make", "win32", [ lib_target, test_C, test_Cpp, test_SearchProcess, test_STL ])
dm            = Compiler::new("dm", "Digital Mars", "make", "", [ lib_target, map_D, test_C, test_Cpp, test_SearchProcess, test_STL ])
icl7_win32    = Compiler::new("icl7", "Intel C/C++ 7", "nmake -nologo", "win32", [ lib_target, map_DLL, test_C, test_Cpp, test_SearchProcess, test_STL ])
icl8_win32    = Compiler::new("icl8", "Intel C/C++ 8", "nmake -nologo", "win32", [ lib_target, map_DLL, test_C, test_Cpp, test_SearchProcess, test_STL ])
ow12_win32    = Compiler::new("ow12", "Open Watcom 1.2", "make", "win32", [ lib_target, test_C, test_SearchProcess ])
vc6           = Compiler::new("vc6", "Visual C++ 6.0", "nmake -nologo", "", [ lib_target, map_DLL, map_COM, test_C, test_COM, test_Cpp, test_SearchProcess, test_STL ])
vc7           = Compiler::new("vc7", "Visual C++ 7.0", "nmake -nologo", "", [ lib_target, map_DLL, map_CS, map_COM, test_C, test_COM, test_Cpp, test_CS, test_SearchProcess, test_STL ])
vc71          = Compiler::new("vc71", "Visual C++ 7.1", "nmake -nologo", "", [ lib_target, map_DLL, map_CS, map_COM, test_C, test_COM, test_Cpp, test_CS, test_SearchProcess, test_STL ])


#compilers = [ bc56, como433_win32, cw8_win32, dm, icl7_win32, icl8_win32, ow12_win32, vc6, vc7, vc71 ]
compilers = [ bc56, como433_win32, cw8_win32, icl7_win32, icl8_win32, ow12_win32, vc6, vc7, vc71 ]

def createMakefile(compiler)
	makefileName = "makefile." + compiler.name + (compiler.suffix.length > 0 ? "." + compiler.suffix : "")
	f = File::new(makefileName, "w")

	maxProjLen = 0
	compiler.projects.each { |project| maxProjLen = project.name.length if project.name.length > maxProjLen }
	
	f << "# ##############################################################################" << "\n"
	f << "# File:         #{makefileName}" << "\n"
	f << "#" << "\n"
	f << "# Purpose:      Makefile for #{compiler.desc}" << "\n"
	f << "#" << "\n"
	f << "# Created:      #{currentDate}" << "\n"
	f << "# Updated:      #{currentDate}" << "\n"
	f << "#" << "\n"
	f << "# Copyright:    Synesis Software Pty Ltd, (c) 2003-2004. All rights reserved." << "\n"
	f << "#" << "\n"
	f << "# Home:         www.synesis.com.au/software" << "\n"
	f << "#" << "\n"
	f << "# ##############################################################################" << "\n"
	f << "\n"
	f << "# ##############################################################################" << "\n"
	f << "# Macros" << "\n"
	f << "" << "\n"
	f << "COMP_TAG			=	#{compiler.name}" << "\n"
	f << "RECLS_DEV			=	." << "\n"
	f << "RECLS_LIBDIR		=	$(RECLS_DEV)\\lib" << "\n"
	f << "RECLS_BINDIR		=	$(RECLS_DEV)\\bin" << "\n"
	f << "RECLS_API_BUILD_DIR	=	$(RECLS_DEV)\\API\\build" << "\n"
	f << "RECLS_API_TEST_DIR	=	$(RECLS_DEV)\\Test" << "\n"
	f << "RECLS_MAPPINGS_DIR	=	$(RECLS_DEV)\\Mappings" << "\n"
	f << "" << "\n"
	f << "\n"
	f << "\n"
	f << "usage: " << "\n"
	f << "	-@echo \"#{compiler.make} [target | clean]\"" << "\n"
	f << "" << "\n"
	f << "target:         " << "".ljust(maxProjLen) << "        \\" << "\n"
	f << "                " << "".ljust(maxProjLen) << "        \\" << "\n"
	compiler.projects.each { |project| f << "            make_" << project.name.ljust(maxProjLen) << "       \\" << "\n" }
	f << "                " << "".ljust(maxProjLen) << "        \\" << "\n"
	f << "\n"
	f << "clean:          " << "".ljust(maxProjLen) << "        \\" << "\n"
	f << "                " << "".ljust(maxProjLen) << "        \\" << "\n"
	compiler.projects.each { |project| f << "            clean_" << project.name.ljust(maxProjLen) << "      \\" << "\n" }
	f << "                " << "".ljust(maxProjLen) << "        \\" << "\n"
	f << "\n"

	compiler.projects.each \
	{ |project|
		f << "make_" << project.name << ":" << "\n"
		f << "\t@echo Making #{project.desc}" << "\n"
		f << "\t@cd "
		bFirst = true
		project.directories.each \
		{ |directory|
			if bFirst
				bFirst = false
			else
				f << Recls::pathNameSeparator
			end
			f << directory
		}
		f << "\n"
		if compiler.suffix.length > 0
			f << "\t@#{compiler.make} -f makefile" << (compiler.suffix.length > 0 ? "." + compiler.suffix : "") << " target" << "\n"
		else
			f << "\t@#{compiler.make}" << " target" << "\n"
		end
		f << "\t@cd "
		for i in 1 .. project.dirDepth
			if i != 1
				f << Recls::pathNameSeparator
			end
			f << ".."
		end
		f << "\n"
		f << "\n"
		
		f << "clean_" << project.name << ":" << "\n"
		f << "\t@echo Cleaning #{project.desc}" << "\n"
		f << "\t@cd "
		bFirst = true
		project.directories.each \
		{ |directory|
			if bFirst
				bFirst = false
			else
				f << Recls::pathNameSeparator
			end
			f << directory
		}
		f << "\n"
		if compiler.suffix.length > 0
			f << "\t@#{compiler.make} -f makefile" << (compiler.suffix.length > 0 ? "." + compiler.suffix : "") << " clean" << "\n"
		else
			f << "\t@#{compiler.make}" << " clean" << "\n"
		end
		f << "\t@cd "
		for i in 1 .. project.dirDepth
			if i != 1
				f << Recls::pathNameSeparator
			end
			f << ".."
		end
		f << "\n"
		f << "\n"
	}

	
	f.close
end

compilers.each { |compiler| createMakefile(compiler) }

